# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import testcase as tc

from tempest.lib import decorators
from tempest import config

from sahara_tempest_plugin.tests.api import base as dp_base

CONF = config.CONF


class PluginsTest(dp_base.BaseDataProcessingTest):
    def _list_all_plugin_names(self):
        """Returns all enabled plugin names.

        It ensures main plugins availability.
        """
        plugins = self.client.list_plugins()['plugins']
        plugins_names = [plugin['name'] for plugin in plugins]
        for enabled_plugin in CONF.data_processing_feature_enabled.plugins:
            self.assertIn(enabled_plugin, plugins_names)

        return plugins_names

    @tc.attr('smoke')
    @decorators.idempotent_id('01a005a3-426c-4c0b-9617-d09475403e09')
    def test_plugin_list(self):
        self._list_all_plugin_names()

    @tc.attr('smoke')
    @decorators.idempotent_id('53cf6487-2cfb-4a6f-8671-97c542c6e901')
    def test_plugin_get(self):
        for plugin_name in self._list_all_plugin_names():
            plugin = self.client.get_plugin(plugin_name)['plugin']
            self.assertEqual(plugin_name, plugin['name'])

            for plugin_version in plugin['versions']:
                detailed_plugin = self.client.get_plugin(plugin_name,
                                                         plugin_version)
                detailed_plugin = detailed_plugin['plugin']
                self.assertEqual(plugin_name, detailed_plugin['name'])

                # check that required image tags contains name and version
                image_tags = detailed_plugin['required_image_tags']
                self.assertIn(plugin_name, image_tags)
                self.assertIn(plugin_version, image_tags)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

from jsonschema import exceptions
import pkg_resources as pkg
import mock
import testtools

from sahara_tests import version
from sahara_tests.scenario import runner


def _create_subprocess_communicate_mock():
    communicate_mock = mock.Mock()
    communicate_mock.communicate.return_value = ["", ""]
    return communicate_mock


class RunnerUnitTest(testtools.TestCase):

    def _isDictContainSubset(self, sub_dictionary, dictionary):
        for key in sub_dictionary:
            if sub_dictionary[key] != dictionary[key]:
                return False
        return True

    def test_set_defaults(self):
        config_without_cred_net = {
            "clusters": [
                {
                    "plugin_name": "vanilla",
                    "plugin_version": "2.7.1",
                    "image": "sahara-vanilla-2.7.1-ubuntu-14.04"
                }
            ]
        }

        expected_default_credential = {
            "credentials": {
                "sahara_url": None,
                "sahara_service_type": "data-processing",
                "ssl_cert": None,
                "ssl_verify": False
            }
        }

        expected_default_network = {
            "network": {
                "type": "neutron",
                "private_network": "private",
                "public_network": "public",
                "auto_assignment_floating_ip": False
            }
        }

        expected_default_cluster = {
            "clusters": [
                {
                    "image": "sahara-vanilla-2.7.1-ubuntu-14.04",
                    "edp_jobs_flow": [],
                    "class_name": "vanilla2_7_1",
                    "plugin_name": "vanilla",
                    "scenario": ['run_jobs', 'scale', 'run_jobs'],
                    "plugin_version": "2.7.1",
                    "retain_resources": False,
                }
            ]
        }

        runner.set_defaults(config_without_cred_net)

        self.assertTrue(self._isDictContainSubset(
            expected_default_credential, config_without_cred_net))
        self.assertTrue(self._isDictContainSubset(
            expected_default_network, config_without_cred_net))
        self.assertTrue(self._isDictContainSubset(
            expected_default_cluster, config_without_cred_net))

        config = {
            "credentials": {
                "os_username": "changed_admin",
                "os_auth_url": "http://127.0.0.1:5000/v2.0",
                "sahara_url": "http://127.0.0.1",
                "os_password": "changed_nova",
                "os_tenant": "changed_admin",
                "ssl_cert": "sahara_tests/unit/scenario/dummy.crt"
            },
            "network": {
                "type": "neutron",
                "private_network": "changed_private",
                "public_network": "changed_public",
                "auto_assignment_floating_ip": True,
            },
            "clusters": [
                {
                    "plugin_name": "vanilla",
                    "plugin_version": "2.7.1",
                    "image": "sahara-vanilla-2.7.1-ubuntu-14.04",
                    "edp_jobs_flow": "test_flow",
                    "retain_resources": True
                }
            ],
            "edp_jobs_flow": {
                "test_flow": [
                    {
                        "type": "Pig",
                        "input_datasource": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/top-todoers/"
                                      "data/input"
                        },
                        "output_datasource": {
                            "type": "hdfs",
                            "destination": "/user/hadoop/edp-output"
                        },
                        "main_lib": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/top-todoers/"
                                      "example.pig"
                        }
                    },
                    {
                        "type": "Java",
                        "additional_libs": [
                            {
                                "type": "database",
                                "source": 'sahara_tests/scenario/defaults/'
                                          'edp-examples/hadoop2/edp-java/'
                                          'hadoop-mapreduce-examples-'
                                          '2.6.0.jars',
                            }],
                        "configs": "edp.java.main_class: org.apache.hadoop."
                                   "examples.QuasiMonteCarlo",
                        "args": [10, 10]
                    },
                ],
                "test_flow2": [
                    {
                        "type": "Java",
                        "additional_libs": [
                            {
                                "type": "database",
                                "source":
                                    "sahara_tests/scenario/defaults/"
                                    "edp-examples/hadoop2/edp-java/hadoop-"
                                    "mapreduce-examples-2.6.0.jars"
                            }],
                        "configs": "edp.java.main_class: org.apache.hadoop."
                                   "examples.QuasiMonteCarlo",
                        "args": [20, 20]
                    }
                ]
            }
        }

        expected_credential = {
            "credentials": {
                "os_username": "changed_admin",
                "os_auth_url": "http://127.0.0.1:5000/v2.0",
                "sahara_url": "http://127.0.0.1",
                "os_password": "changed_nova",
                "os_tenant": "changed_admin",
                "sahara_service_type": "data-processing",
                "ssl_cert": "sahara_tests/unit/scenario/dummy.crt",
                "ssl_verify": False
            },
        }

        expected_network = {
            "network": {
                "type": "neutron",
                "private_network": "changed_private",
                "public_network": "changed_public",
                "auto_assignment_floating_ip": True,
            }
        }

        expected_cluster = {
            "clusters": [
                {
                    "plugin_name": "vanilla",
                    "plugin_version": "2.7.1",
                    "image": "sahara-vanilla-2.7.1-ubuntu-14.04",
                    "retain_resources": True,
                    'edp_jobs_flow': [
                        {
                            'main_lib': {
                                'source': 'sahara_tests/scenario/defaults/'
                                          'edp-examples/edp-pig/'
                                          'top-todoers/example.pig',
                                'type': 'swift'
                            },
                            'type': 'Pig',
                            'input_datasource': {
                                'source': 'sahara_tests/scenario/defaults/'
                                          'edp-examples/edp-pig/'
                                          'top-todoers/data/input',
                                'type': 'swift'
                            },
                            'output_datasource': {
                                'type': 'hdfs',
                                'destination': '/user/hadoop/edp-output'
                            }
                        },
                        {
                            'args': [10, 10],
                            'configs': 'edp.java.main_class: org.apache.'
                                       'hadoop.examples.QuasiMonteCarlo',
                            'type': 'Java',
                            'additional_libs': [
                                {
                                    'source': 'sahara_tests/scenario/defaults/'
                                              'edp-examples/hadoop2/edp-java/'
                                              'hadoop-mapreduce-examples-'
                                              '2.6.0.jars',
                                    'type': 'database'
                                }]
                        }
                    ],
                    "scenario": ['run_jobs', 'scale', 'run_jobs'],
                    "class_name": "vanilla2_7_1"
                }],
        }

        runner.set_defaults(config)

        self.assertTrue(self._isDictContainSubset(
            expected_credential, config))
        self.assertTrue(self._isDictContainSubset(
            expected_network, config))
        self.assertTrue(self._isDictContainSubset(
            expected_cluster, config))

    @mock.patch('sys.exit', return_value=None)
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    def test_runner_main(self, mock_sub, mock_sys):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    'sahara_tests/unit/scenario/vanilla2_7_1.yaml']
        runner.main()

    @mock.patch('sys.exit', return_value=None)
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    def test_runner_template_missing_varfile(self, mock_sub, mock_sys):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    'sahara_tests/unit/scenario/vanilla2_7_1.yaml.mako']
        self.assertRaises(NameError, runner.main)

    @mock.patch('sys.exit', return_value=None)
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    def test_runner_template_wrong_varfile(self, mock_sub, mock_sys):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_nodefault.ini',
                    'sahara_tests/unit/scenario/vanilla2_7_1.yaml.mako']
        self.assertRaises(NameError, runner.main)

    @mock.patch('sys.exit', return_value=None)
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    def test_runner_template_incomplete_varfile(self, mock_sub, mock_sys):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_incomplete.ini',
                    'sahara_tests/unit/scenario/vanilla2_7_1.yaml.mako']
        self.assertRaises(NameError, runner.main)

    @mock.patch('sys.exit', return_value=None)
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    def test_runner_template_working(self, mock_sub, mock_sys):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    'sahara_tests/unit/scenario/vanilla2_7_1.yaml.mako']
        runner.main()

    @mock.patch('sys.exit', return_value=None)
    def test_runner_validate(self, mock_sys):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '--validate',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    'sahara_tests/unit/scenario/vanilla2_7_1.yaml.mako']
        runner.main()

    @mock.patch('sahara_tests.scenario.validation.validate')
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_runner_from_args(self, mock_sys, mock_sub, mock_validate):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    'sahara_tests/unit/scenario/vanilla2_7_1.yaml.mako',
                    '--os-username', 'demo', '--os-password', 'demopwd',
                    '--os-project-name', 'demo', '--os-auth-url', 'localhost',
                    '--args', 'network_type:neutron',
                    'network_private_name:private',
                    'network_public_name:public',
                    'vanilla_26_image:hadoop_2_6_latest',
                    'ci_flavor_id:2']
        runner.main()
        expected = {
            'os_username': 'demo',
            'os_password': 'demopwd',
            'os_tenant': 'demo',
            'os_auth_url': 'localhost/v2.0'
        }
        self.assertTrue(self._isDictContainSubset(
            expected, mock_validate.call_args_list[0][0][0]['credentials']))

    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_replace_value_args(self, mock_sys, mock_sub):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    'sahara_tests/unit/scenario/vanilla2_7_1.yaml.mako',
                    '--args',
                    'network_type:test']
        with testtools.ExpectedException(exceptions.ValidationError):
            runner.main()

    @mock.patch('sahara_tests.scenario.validation.validate')
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_default_templates_non_plugin(self, mock_sys, mock_sub,
                                          mock_validate):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    '-p', 'transient', '--os-username', 'demo',
                    '--os-password', 'demopwd',
                    '--os-project-name', 'demo',
                    '--os-auth-url', 'http://127.0.0.1:5000/v2']
        runner.main()

    @mock.patch('sahara_tests.scenario.validation.validate')
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_default_templates_negative(self, mock_sys, mock_sub,
                                        mock_validate):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    '-p', 'vanilla', '--os-username', 'demo',
                    '--os-password', 'demopwd',
                    '--os-project-name', 'demo',
                    '--os-auth-url', 'http://127.0.0.1:5000/v2']
        with testtools.ExpectedException(ValueError):
            runner.main()

    @mock.patch('sahara_tests.scenario.validation.validate')
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_default_templates(self, mock_sys, mock_sub, mock_validate):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    '-p', 'spark', '-v', '1.6.0', '-r', 'newton',
                    '--os-username', 'demo', '--os-password', 'demopwd',
                    '--os-project-name', 'demo',
                    '--os-auth-url', 'http://127.0.0.1:5000/v2']
        runner.main()
        self.assertEqual('spark',
                         mock_validate.call_args[0][0]['clusters'][0][
                             'plugin_name'])
        self.assertEqual('1.6.0',
                         mock_validate.call_args[0][0]['clusters'][0][
                             'plugin_version'])

    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_count(self, mock_sys, mock_sub):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    '-p', 'spark', '-v', '1.6.0', '--release', 'newton',
                    '--count', '4',
                    '--os-username', 'demo', '--os-password', 'demopwd',
                    '--os-project-name', 'demo',
                    '--os-auth-url', 'http://127.0.0.1:5000/v2']
        runner.main()

    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_run_dir(self, mock_sys, mock_sub):
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    'sahara_tests/scenario/defaults/newton',
                    'sahara_tests/scenario/defaults/edp.yaml.mako',
                    '--os-username', 'demo', '--os-password', 'demopwd',
                    '--os-project-name', 'demo',
                    '--os-auth-url', 'http://127.0.0.1:5000/v2', '--args',
                    'ambari_22_image:ambari', 'fake_plugin_image:fake',
                    'mapr_510mrv2_image:mapr', 'mapr_520mrv2_image:mapr',
                    'cdh_570_image:cdh', 'spark_160_image:spark',
                    'storm_101_image:storm']
        runner.main()

    @mock.patch('sahara_tests.scenario.validation.validate')
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_credentials_envvars(self, mock_sys, mock_sub, mock_validate):
        username = os.environ.get('OS_USERNAME', '')
        password = os.environ.get('OS_PASSWORD', '')
        auth_url = os.environ.get('OS_AUTHURL', '')
        project_name = os.environ.get('OS_PROJECT_NAME', '')
        os.environ['OS_USERNAME'] = 'demo_env'
        os.environ['OS_PASSWORD'] = 'demopwd_env'
        os.environ['OS_AUTH_URL'] = 'http://localhost:5000/v2.0'
        os.environ['OS_PROJECT_NAME'] = 'project_env'
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    '-p', 'spark', '-v', '1.3.1', '--release', 'liberty',
                    '--count', '4']
        runner.main()
        expected = {
            'os_username': 'demo_env',
            'os_password': 'demopwd_env',
            'os_tenant': 'project_env',
            'os_auth_url': 'http://localhost:5000/v2.0'
        }
        self.assertTrue(self._isDictContainSubset(
            expected, mock_validate.call_args_list[0][0][0]['credentials']))

        os.environ['OS_USERNAME'] = username
        os.environ['OS_PASSWORD'] = password
        os.environ['OS_AUTHURL'] = auth_url
        os.environ['OS_PROJECT_NAME'] = project_name

    @mock.patch('sahara_tests.scenario.validation.validate')
    @mock.patch('subprocess.Popen',
                return_value=_create_subprocess_communicate_mock())
    @mock.patch('sys.exit', return_value=None)
    def test_credentials_clouds(self, mock_sys, mock_sub, mock_validate):
        unit_dir = pkg.resource_filename(version.version_info.package,
                                         'unit')
        os_client_config_file = os.environ.get('OS_CLIENT_CONFIG_FILE', '')
        os.environ['OS_CLIENT_CONFIG_FILE'] = os.path.join(unit_dir,
                                                           'scenario',
                                                           'clouds.yaml')
        sys.argv = ['sahara_tests/scenario/runner.py',
                    '-V',
                    'sahara_tests/unit/scenario/templatevars_complete.ini',
                    '-p', 'spark', '-v', '1.0.0', '--count', '4',
                    '--os-cloud', 'scenario_test']
        runner.main()
        expected = {
            'os_username': 'demo_cloud',
            'os_password': 'demo_cloud_pwd',
            'os_tenant': 'demo_cloud_project',
            'os_auth_url': 'http://localhost:5000/v2.0'
        }
        self.assertTrue(self._isDictContainSubset(
            expected, mock_validate.call_args_list[0][0][0]['credentials']))
        os.environ['OS_CLIENT_CONFIG_FILE'] = os_client_config_file




# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import sys

from pyspark import SparkContext
from pyspark.streaming.kafka import KafkaUtils
from pyspark.streaming import StreamingContext


def main():
    if len(sys.argv) != 4:
        print("Usage: kafka_wordcount.py <zk> <topic> <timeout>",
              file=sys.stderr)
        exit(-1)

    sc = SparkContext(appName="PythonStreamingKafkaWordCount")
    ssc = StreamingContext(sc, 1)
    timeout = None
    if len(sys.argv) == 4:
        zk, topic, timeout = sys.argv[1:]
        timeout = int(timeout)
    else:
        zk, topic = sys.argv[1:]
    kvs = KafkaUtils.createStream(
        ssc, zk, "spark-streaming-consumer", {topic: 1})
    lines = kvs.map(lambda x: x[1])
    counts = lines.flatMap(lambda line: (line.split(" "))
                           .map(lambda word: (word, 1))
                           .reduceByKey(lambda a, b: a + b))
    counts.pprint()
    kwargs = {}
    if timeout:
        kwargs['timeout'] = timeout
    ssc.start()
    ssc.awaitTermination(**kwargs)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import testcase as tc

from tempest.lib import decorators
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.api import base as dp_base


class ClusterTemplateTest(dp_base.BaseDataProcessingTest):
    # Link to the API documentation is https://developer.openstack.org/
    # api-ref/data-processing/#cluster-templates

    @classmethod
    def skip_checks(cls):
        super(ClusterTemplateTest, cls).skip_checks()
        if cls.default_plugin is None:
            raise cls.skipException("No Sahara plugins configured")

    @classmethod
    def resource_setup(cls):
        super(ClusterTemplateTest, cls).resource_setup()

        # pre-define a node group templates
        node_group_template_w = cls.get_node_group_template('worker1')
        if node_group_template_w is None:
            raise dp_base.InvalidSaharaTestConfiguration(
                message="No known Sahara plugin was found")

        node_group_template_w['name'] = data_utils.rand_name(
            'sahara-ng-template')
        resp_body = cls.create_node_group_template(**node_group_template_w)
        node_group_template_id = resp_body['id']
        configured_node_group_templates = {'worker1': node_group_template_id}

        cls.full_cluster_template = cls.get_cluster_template(
            configured_node_group_templates)

        # create cls.cluster_template variable to use for comparison to cluster
        # template response body. The 'node_groups' field in the response body
        # has some extra info that post body does not have. The 'node_groups'
        # field in the response body is something like this
        #
        #   'node_groups': [
        #       {
        #           'count': 3,
        #           'name': 'worker-node',
        #           'volume_mount_prefix': '/volumes/disk',
        #           'created_at': '2014-05-21 14:31:37',
        #           'updated_at': None,
        #           'floating_ip_pool': None,
        #           ...
        #       },
        #       ...
        #   ]
        cls.cluster_template = cls.full_cluster_template.copy()
        del cls.cluster_template['node_groups']

    def _create_cluster_template(self, template_name=None):
        """Creates Cluster Template with optional name specified.

        It creates template, ensures template name and response body.
        Returns id and name of created template.
        """
        if not template_name:
            # generate random name if it's not specified
            template_name = data_utils.rand_name('sahara-cluster-template')

        # create cluster template
        resp_body = self.create_cluster_template(template_name,
                                                 **self.full_cluster_template)

        # ensure that template created successfully
        self.assertEqual(template_name, resp_body['name'])
        self.assertDictContainsSubset(self.cluster_template, resp_body)

        return resp_body['id'], template_name

    @tc.attr('smoke')
    @decorators.idempotent_id('3525f1f1-3f9c-407d-891a-a996237e728b')
    def test_cluster_template_create(self):
        self._create_cluster_template()

    @tc.attr('smoke')
    @decorators.idempotent_id('7a161882-e430-4840-a1c6-1d928201fab2')
    def test_cluster_template_list(self):
        template_info = self._create_cluster_template()

        # check for cluster template in list
        templates = self.client.list_cluster_templates()['cluster_templates']
        templates_info = [(template['id'], template['name'])
                          for template in templates]
        self.assertIn(template_info, templates_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('2b75fe22-f731-4b0f-84f1-89ab25f86637')
    def test_cluster_template_get(self):
        template_id, template_name = self._create_cluster_template()

        # check cluster template fetch by id
        template = self.client.get_cluster_template(template_id)
        template = template['cluster_template']
        self.assertEqual(template_name, template['name'])
        self.assertDictContainsSubset(self.cluster_template, template)

    @tc.attr('smoke')
    @decorators.idempotent_id('ff1fd989-171c-4dd7-91fd-9fbc71b09675')
    def test_cluster_template_delete(self):
        template_id, _ = self._create_cluster_template()

        # delete the cluster template by id
        self.client.delete_cluster_template(template_id)
        get_resource = self.client.get_cluster_template
        self.wait_for_resource_deletion(template_id, get_resource)

        templates = self.client.list_cluster_templates()['cluster_templates']
        templates_info = [template['id']
                          for template in templates]
        self.assertNotIn(template_id, templates_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('40235aa0-cd4b-494a-9c12-2d0e8a92157a')
    def test_cluster_template_update(self):
        template_id, _ = self._create_cluster_template()

        new_template_name = data_utils.rand_name('sahara-cluster-template')
        body = {'name': new_template_name}
        updated_template = self.client.update_cluster_template(template_id,
                                                               **body)
        updated_template = updated_template['cluster_template']

        self.assertEqual(new_template_name, updated_template['name'])



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import testcase as tc

from tempest.lib import decorators
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.api import base as dp_base


class JobBinaryTest(dp_base.BaseDataProcessingTest):
    # Link to the API documentation is https://developer.openstack.org/
    # api-ref/data-processing/#job-binaries

    @classmethod
    def resource_setup(cls):
        super(JobBinaryTest, cls).resource_setup()
        cls.swift_job_binary_with_extra = {
            'url': 'swift://sahara-container.sahara/example.jar',
            'description': 'Test job binary',
            'extra': {
                'user': cls.os_primary.credentials.username,
                'password': cls.os_primary.credentials.password
            }
        }
        # Create extra cls.swift_job_binary variable to use for comparison to
        # job binary response body because response body has no 'extra' field.
        cls.swift_job_binary = cls.swift_job_binary_with_extra.copy()
        del cls.swift_job_binary['extra']

        name = data_utils.rand_name('sahara-internal-job-binary')
        cls.job_binary_data = 'Some script may be data'
        job_binary_internal = (
            cls.create_job_binary_internal(name, cls.job_binary_data))
        cls.internal_db_job_binary = {
            'url': 'internal-db://%s' % job_binary_internal['id'],
            'description': 'Test job binary',
        }

    def _create_job_binary(self, binary_body, binary_name=None):
        """Creates Job Binary with optional name specified.

        It creates a link to data (jar, pig files, etc.), ensures job binary
        name and response body. Returns id and name of created job binary.
        Data may not exist when using Swift as data storage.
        In other cases data must exist in storage.
        """
        if not binary_name:
            # generate random name if it's not specified
            binary_name = data_utils.rand_name('sahara-job-binary')

        # create job binary
        resp_body = self.create_job_binary(binary_name, **binary_body)

        # ensure that binary created successfully
        self.assertEqual(binary_name, resp_body['name'])
        if 'swift' in binary_body['url']:
            binary_body = self.swift_job_binary
        self.assertDictContainsSubset(binary_body, resp_body)

        return resp_body['id'], binary_name

    @tc.attr('smoke')
    @decorators.idempotent_id('c00d43f8-4360-45f8-b280-af1a201b12d3')
    def test_swift_job_binary_create(self):
        self._create_job_binary(self.swift_job_binary_with_extra)

    @tc.attr('smoke')
    @decorators.idempotent_id('f8809352-e79d-4748-9359-ce1efce89f2a')
    def test_swift_job_binary_list(self):
        binary_info = self._create_job_binary(self.swift_job_binary_with_extra)

        # check for job binary in list
        binaries = self.client.list_job_binaries()['binaries']
        binaries_info = [(binary['id'], binary['name']) for binary in binaries]
        self.assertIn(binary_info, binaries_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('2d4a670f-e8f1-413c-b5ac-50c1bfe9e1b1')
    def test_swift_job_binary_get(self):
        binary_id, binary_name = (
            self._create_job_binary(self.swift_job_binary_with_extra))

        # check job binary fetch by id
        binary = self.client.get_job_binary(binary_id)['job_binary']
        self.assertEqual(binary_name, binary['name'])
        self.assertDictContainsSubset(self.swift_job_binary, binary)

    @tc.attr('smoke')
    @decorators.idempotent_id('9b0e8f38-04f3-4616-b399-cfa7eb2677ed')
    def test_swift_job_binary_delete(self):
        binary_id, _ = (
            self._create_job_binary(self.swift_job_binary_with_extra))

        # delete the job binary by id
        self.client.delete_job_binary(binary_id)

    @tc.attr('smoke')
    @decorators.idempotent_id('63662f6d-8291-407e-a6fc-f654522ebab6')
    def test_internal_db_job_binary_create(self):
        self._create_job_binary(self.internal_db_job_binary)

    @tc.attr('smoke')
    @decorators.idempotent_id('38731e7b-6d9d-4ffa-8fd1-193c453e88b1')
    def test_internal_db_job_binary_list(self):
        binary_info = self._create_job_binary(self.internal_db_job_binary)

        # check for job binary in list
        binaries = self.client.list_job_binaries()['binaries']
        binaries_info = [(binary['id'], binary['name']) for binary in binaries]
        self.assertIn(binary_info, binaries_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('1b32199b-c3f5-43e1-a37a-3797e57b7066')
    def test_internal_db_job_binary_get(self):
        binary_id, binary_name = (
            self._create_job_binary(self.internal_db_job_binary))

        # check job binary fetch by id
        binary = self.client.get_job_binary(binary_id)['job_binary']
        self.assertEqual(binary_name, binary['name'])
        self.assertDictContainsSubset(self.internal_db_job_binary, binary)

    @tc.attr('smoke')
    @decorators.idempotent_id('3c42b0c3-3e03-46a5-adf0-df0650271a4e')
    def test_internal_db_job_binary_delete(self):
        binary_id, _ = self._create_job_binary(self.internal_db_job_binary)

        # delete the job binary by id
        self.client.delete_job_binary(binary_id)
        self.wait_for_resource_deletion(binary_id, self.client.get_job_binary)

        binaries = self.client.list_job_binaries()['binaries']
        binaries_ids = [binary['id'] for binary in binaries]
        self.assertNotIn(binary_id, binaries_ids)

    @tc.attr('smoke')
    @decorators.idempotent_id('d5d47659-7e2c-4ea7-b292-5b3e559e8587')
    def test_job_binary_get_data(self):
        binary_id, _ = self._create_job_binary(self.internal_db_job_binary)

        # get data of job binary by id
        _, data = self.client.get_job_binary_data(binary_id)
        self.assertEqual(data, self.job_binary_data)



#!/usr/bin/env python

# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function
import argparse
import collections
import os
import shutil
import subprocess
import sys
import tempfile

from oslo_utils import fileutils
import os_client_config
import pkg_resources as pkg
import six
import yaml

from sahara_tests.scenario import utils
from sahara_tests.scenario import validation
from sahara_tests import version


def set_defaults(config):
    # set up credentials
    config['credentials'] = config.get('credentials', {})
    creds = config['credentials']
    creds.setdefault('sahara_service_type', 'data-processing')
    creds['sahara_url'] = creds.get('sahara_url', None)
    creds['ssl_verify'] = creds.get('ssl_verify', False)
    creds['ssl_cert'] = creds.get('ssl_cert', None)

    # set up network
    config['network'] = config.get('network', {})
    net = config['network']
    net['type'] = net.get('type', 'neutron')
    net['private_network'] = net.get('private_network', 'private')
    net['auto_assignment_floating_ip'] = net.get('auto_assignment_floating_ip',
                                                 False)
    net['public_network'] = net.get('public_network', 'public')

    default_scenario = ['run_jobs', 'scale', 'run_jobs']

    # set up tests parameters
    for testcase in config['clusters']:
        testcase['class_name'] = "".join([
            testcase['plugin_name'],
            testcase['plugin_version'].replace('.', '_')])
        testcase['retain_resources'] = testcase.get('retain_resources', False)
        testcase['scenario'] = testcase.get('scenario', default_scenario)
        if isinstance(testcase.get('edp_jobs_flow'), six.string_types):
            testcase['edp_jobs_flow'] = [testcase['edp_jobs_flow']]
        edp_jobs_flow = []
        for edp_flow in testcase.get('edp_jobs_flow', []):
            edp_jobs_flow.extend(config.get('edp_jobs_flow',
                                            {}).get(edp_flow))
        testcase['edp_jobs_flow'] = edp_jobs_flow


def recursive_walk(directory):
    list_of_files = []
    for file in os.listdir(directory):
        path = os.path.join(directory, file)
        if os.path.isfile(path):
            list_of_files.append(path)
        else:
            list_of_files += recursive_walk(path)
    return list_of_files


def valid_count(value):
    try:
        ivalue = int(value)
        if ivalue <= 0:
            raise ValueError
    except ValueError:
        raise argparse.ArgumentTypeError("%s is an invalid value of count. "
                                         "Value must be int and > 0. " % value)
    return ivalue


def parse_args(array):
    args = collections.OrderedDict()
    for pair in array:
        arg_dict = pair.split(':')
        if len(arg_dict) < 2:
            return args
        args[arg_dict[0]] = "\'%s\'" % arg_dict[1]
    return args


def get_base_parser():
    parser = argparse.ArgumentParser(description="Scenario tests runner.")

    parser.add_argument('scenario_arguments', help="Path to scenario files",
                        nargs='*', default=[])
    parser.add_argument('--variable_file', '-V', default='', nargs='?',
                        help='Path to the file with template variables')
    parser.add_argument('--verbose', default=False, action='store_true',
                        help='Increase output verbosity')
    parser.add_argument('--validate', default=False, action='store_true',
                        help='Validate yaml-files, tests will not be runned')
    parser.add_argument('--args', default='', nargs='+',
                        help='Pairs of arguments key:value')
    parser.add_argument('--plugin', '-p', default=None, nargs='?',
                        help='Specify plugin name')
    parser.add_argument('--plugin_version', '-v', default=None,
                        nargs='?', help='Specify plugin version')
    parser.add_argument('--release', '-r', default=None,
                        nargs='?', help='Specify Sahara release')
    parser.add_argument('--report', default=False, action='store_true',
                        help='Write results of test to file')
    parser.add_argument('--count', default=1, nargs='?', type=valid_count,
                        help='Specify count of runs current cases.')
    return parser


def get_scenario_files(scenario_arguments):
    files = []
    for scenario_argument in scenario_arguments:
        if os.path.isdir(scenario_argument):
            files += recursive_walk(scenario_argument)
        if os.path.isfile(scenario_argument):
            files.append(scenario_argument)

    return files


def main():
    # parse args
    cloud_config = os_client_config.OpenStackConfig()
    parser = get_base_parser()
    cloud_config.register_argparse_arguments(parser, sys.argv)
    args = parser.parse_args()

    scenario_arguments = args.scenario_arguments
    variable_file = args.variable_file
    verbose_run = args.verbose
    scenario_args = parse_args(args.args)
    plugin = args.plugin
    version = args.plugin_version
    release = args.release
    report = args.report
    count = args.count

    auth_values = utils.get_auth_values(cloud_config, args)

    scenario_arguments = utils.get_default_templates(plugin, version, release,
                                                     scenario_arguments)

    files = get_scenario_files(scenario_arguments)

    template_variables = utils.get_templates_variables(files, variable_file,
                                                       verbose_run,
                                                       scenario_args,
                                                       auth_values)

    params_for_login = {'credentials': auth_values}
    config = utils.generate_config(files, template_variables, params_for_login,
                                   verbose_run)

    # validate config
    validation.validate(config)

    if args.validate:
        return

    set_defaults(config)
    credentials = config['credentials']
    network = config['network']
    testcases = config['clusters']
    for case in range(count - 1):
        testcases.extend(config['clusters'])

    test_dir_path = utils.create_testcase_file(testcases, credentials, network,
                                               report)

    # run tests
    concurrency = config.get('concurrency')
    testr_runner_exit_code = utils.run_tests(concurrency, test_dir_path)
    sys.exit(testr_runner_exit_code)

if __name__ == '__main__':
    main()



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import testcase as tc

from tempest.lib import decorators
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.api import base as dp_base


class JobTest(dp_base.BaseDataProcessingTest):
    # NOTE: Link to the API documentation: https://developer.openstack.org/
    # api-ref/data-processing/#jobs

    @classmethod
    def resource_setup(cls):
        super(JobTest, cls).resource_setup()
        # create job binary
        job_binary = {
            'name': data_utils.rand_name('sahara-job-binary'),
            'url': 'swift://sahara-container.sahara/example.jar',
            'description': 'Test job binary',
            'extra': {
                'user': cls.os_primary.credentials.username,
                'password': cls.os_primary.credentials.password
            }
        }
        resp_body = cls.create_job_binary(**job_binary)
        job_binary_id = resp_body['id']

        cls.job = {
            'job_type': 'Pig',
            'mains': [job_binary_id]
        }

    def _create_job(self, job_name=None):
        """Creates Job with optional name specified.

        It creates job and ensures job name. Returns id and name of created
        job.
        """
        if not job_name:
            # generate random name if it's not specified
            job_name = data_utils.rand_name('sahara-job')

        # create job
        resp_body = self.create_job(job_name, **self.job)

        # ensure that job created successfully
        self.assertEqual(job_name, resp_body['name'])

        return resp_body['id'], job_name

    @tc.attr('smoke')
    @decorators.idempotent_id('8cf785ca-adf4-473d-8281-fb9a5efa3073')
    def test_job_create(self):
        self._create_job()

    @tc.attr('smoke')
    @decorators.idempotent_id('41e253fe-b02a-41a0-b186-5ff1f0463ba3')
    def test_job_list(self):
        job_info = self._create_job()

        # check for job in list
        jobs = self.client.list_jobs()['jobs']
        jobs_info = [(job['id'], job['name']) for job in jobs]
        self.assertIn(job_info, jobs_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('3faf17fa-bc94-4a60-b1c3-79e53674c16c')
    def test_job_get(self):
        job_id, job_name = self._create_job()

        # check job fetch by id
        job = self.client.get_job(job_id)['job']
        self.assertEqual(job_name, job['name'])

    @tc.attr('smoke')
    @decorators.idempotent_id('dff85e62-7dda-4ad8-b1ee-850adecb0c6e')
    def test_job_delete(self):
        job_id, _ = self._create_job()

        # delete the job by id
        self.client.delete_job(job_id)
        self.wait_for_resource_deletion(job_id, self.client.get_job)

        jobs = self.client.list_jobs()['jobs']
        jobs_ids = [job['id'] for job in jobs]
        self.assertNotIn(job_id, jobs_ids)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.cli import base


class SaharaNodeGroupCLITest(base.ClientTestBase):

    def openstack_node_group_template_list(self):
        self.assertTableStruct(
            self.listing_result('node group template list'), [
                'Name',
                'Id',
                'Plugin version',
                'Plugin name'
            ])

    def openstack_node_group_template_create(self, ng_type, flavor_id):
        plugin = self.get_default_plugin()
        id_net_pool = self.find_id_of_pool()
        node_group_name = data_utils.rand_name(ng_type)
        flags = ("%(ngt_name)s %(plugin)s %(plugin-version)s "
                 "%(processes)s %(flavor)s %(floating-pool)s"
                 % {'floating-pool': ' --floating-ip-pool %s' % id_net_pool,
                    'flavor': ' --flavor %s' % flavor_id,
                    'processes': ' --processes datanode',
                    'plugin-version': ' --plugin-version %s'
                                      % plugin['Versions'],
                    'plugin': ' --plugin %s' % plugin['Name'],
                    'ngt_name': '--name %s' % node_group_name})
        self.assertTableStruct(
            self.listing_result('node group template create %s' % flags), [
                'Field',
                'Value'
            ])
        return node_group_name

    def openstack_node_group_template_show(self, node_group_name):
        self.find_in_listing(
            self.listing_result('node group template show %s'
                                % node_group_name),
            node_group_name)

    def openstack_node_group_template_update(self, node_group_name,
                                             update_field=None):
        """Update node group template with necessary parameters
        Args:
            node_group_name (str): name of node group to update
            update_field (str): param how to update the node group. Using
            this arg, there are several available updates of node group:
            name, public/private, protected/unprotected
        """
        cmd = 'node group template'
        if update_field == 'name':
            new_node_group_name = self.update_resource_value(cmd,
                                                             node_group_name,
                                                             '--name')
            return new_node_group_name
        elif update_field in ('protected', 'unprotected'):
            # here we check only updating with public/protected flags for now
            update_cmd = 'update %s --%s' % (node_group_name, update_field)
            result = self.listing_result('%s %s' % (cmd, update_cmd))
            is_protected_value = str(update_field == 'protected')
            self.find_in_listing(result, is_protected_value, 'is protected')
            self.assertTableStruct(result, ['Field', 'Value'])

    def openstack_node_group_template_delete(self, node_group_name):
        self.check_if_delete('node group template', node_group_name)

    def negative_delete_removed_node_group(self, node_group_name):
        """Test to remove already deleted node group template"""
        command_to_execute = 'node group template delete'
        self.check_negative_scenarios(base.TEMPEST_ERROR_MESSAGE,
                                      command_to_execute,
                                      node_group_name)

    def negative_try_to_delete_protected_node_group(self, node_group_name):
        """Test to delete protected node group template"""
        self.openstack_node_group_template_update(node_group_name,
                                                  update_field='protected')
        error_message = ("NodeGroupTemplate with id '%s' could not be deleted "
                         "because it's marked as protected"
                         % self._get_resource_id('node group template',
                                                 node_group_name))
        command_to_execute = 'node group template delete'
        self.check_negative_scenarios(error_message,
                                      command_to_execute,
                                      node_group_name)
        self.openstack_node_group_template_update(node_group_name,
                                                  update_field='unprotected')

    def filter_node_group_list_with_plugin(self):
        """Test to filter node group templates with the plugin"""
        plugins_list = self.listing_result('plugin list')
        plugins_names = [p['Name'] for p in plugins_list]
        if len(plugins_names) == 0:
            raise self.skipException('No plugin to filter node group')
        filter_cmd = self.listing_result(
            'node group template list --plugin %s' % plugins_names[0])
        for ng in filter_cmd:
            self.assertEqual(plugins_names[0], ng['Plugin name'])
        self.assertTableStruct(filter_cmd, [
            'Name',
            'Id',
            'Plugin version',
            'Plugin name'
        ])



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.clients import base


class NodeGroupTemplateTest(base.BaseDataProcessingTest):
    def _check_create_node_group_template(self):
        template_name = data_utils.rand_name('sahara-ng-template')

        # create node group template
        resp_body = self.create_node_group_template(template_name,
                                                    **self.worker_template)
        # check that template created successfully
        self.assertEqual(template_name, resp_body.name)
        self.assertDictContainsSubset(self.worker_template,
                                      resp_body.__dict__)

        return resp_body.id, template_name

    def _check_node_group_template_list(self, template_id, template_name):
        # check for node group template in list
        template_list = self.client.node_group_templates.list()
        templates_info = [(template.id, template.name)
                          for template in template_list]
        self.assertIn((template_id, template_name), templates_info)

    def _check_node_group_template_get(self, template_id, template_name):
        # check node group template fetch by id
        template = self.client.node_group_templates.get(
            template_id)
        self.assertEqual(template_name, template.name)
        self.assertDictContainsSubset(self.worker_template,
                                      template.__dict__)

    def _check_node_group_template_update(self, template_id):
        values = {
            'name': data_utils.rand_name('updated-sahara-ng-template'),
            'description': 'description',
            'volumes_per_node': 2,
            'volumes_size': 2,
        }

        resp_body = self.client.node_group_templates.update(template_id,
                                                            **values)
        # check that template updated successfully
        self.assertDictContainsSubset(values,
                                      resp_body.__dict__)

    def _check_node_group_template_delete(self, template_id):
        # delete node group template by id
        self.client.node_group_templates.delete(template_id)

        # check that node group really deleted
        templates = self.client.node_group_templates.list()
        self.assertNotIn(template_id, [template.id for template in templates])

    def test_node_group_templates(self):
        template_id, template_name = self._check_create_node_group_template()
        self._check_node_group_template_list(template_id, template_name)
        self._check_node_group_template_get(template_id, template_name)
        self._check_node_group_template_update(template_id)
        self._check_node_group_template_delete(template_id)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


class Defaults(object):
    def __init__(self, config):
        self.timeout_check_transient = config.get('timeout_check_transient',
                                                  300)
        self.timeout_delete_resource = config.get('timeout_delete_resource',
                                                  300)
        self.timeout_poll_cluster_status = config.get(
            'timeout_poll_cluster_status', 3600)
        self.timeout_poll_jobs_status = config.get('timeout_poll_jobs_status',
                                                   1800)

    @classmethod
    def init_defaults(cls, config):
        if not hasattr(cls, 'instance'):
            cls.instance = Defaults(config)
        return cls.instance



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from os import remove

from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.cli import base

GET_CONFIG_RESULT = '"%s" job configs were saved in "%s"file'


class SaharaJobTypeCLITest(base.ClientTestBase):

    def openstack_job_type_list(self):
        self.assertTableStruct(self.listing_result('job type list'), [
            'Name',
            'Plugins'
        ])

    def openstack_job_type_configs_get(self, flag=None):
        list_job_type = self.listing_result('job type list')
        job_type_names = [p['Name'] for p in list_job_type]
        if len(job_type_names) == 0:
            raise self.skipException('No job types to get configs')
        cmd = 'job type configs get'
        type_name = job_type_names[0]
        if flag == 'file':
            file_name = data_utils.rand_name('filename')
            cmd = '%s --%s %s %s' % (cmd, flag, file_name, type_name)
        else:
            file_name = type_name
            cmd = '%s %s' % (cmd, type_name)
        result = self.openstack('dataprocessing %s' % cmd)
        expected_result = GET_CONFIG_RESULT % (type_name, file_name)
        self.assertEqual(result, expected_result)
        remove(file_name)

    def filter_job_type_in_list(self):
        """Filter job types list with --column flag"""
        list_job_types = self.listing_result('job type list --column Name')
        self.assertTableStruct(list_job_types, ['Name'])



# Copyright (c) 2013 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_serialization import jsonutils as json

from tempest.lib.common import rest_client


class DataProcessingClient(rest_client.RestClient):

    def _request_and_check_resp(self, request_func, uri, resp_status):
        """Make a request and check response status code.

        It returns a ResponseBody.
        """
        resp, body = request_func(uri)
        self.expected_success(resp_status, resp.status)
        return rest_client.ResponseBody(resp, body)

    def _request_and_check_resp_data(self, request_func, uri, resp_status):
        """Make a request and check response status code.

        It returns pair: resp and response data.
        """
        resp, body = request_func(uri)
        self.expected_success(resp_status, resp.status)
        return resp, body

    def _request_check_and_parse_resp(self, request_func, uri,
                                      resp_status, *args, **kwargs):
        """Make a request, check response status code and parse response body.

        It returns a ResponseBody.
        """
        headers = {'Content-Type': 'application/json'}
        resp, body = request_func(uri, headers=headers, *args, **kwargs)
        self.expected_success(resp_status, resp.status)
        body = json.loads(body)
        return rest_client.ResponseBody(resp, body)

    def list_node_group_templates(self):
        """List all node group templates for a user."""

        uri = 'node-group-templates'
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def get_node_group_template(self, tmpl_id):
        """Returns the details of a single node group template."""

        uri = 'node-group-templates/%s' % tmpl_id
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def create_node_group_template(self, name, plugin_name, hadoop_version,
                                   node_processes, flavor_id,
                                   node_configs=None, **kwargs):
        """Creates node group template with specified params.

        It supports passing additional params using kwargs and returns created
        object.
        """
        uri = 'node-group-templates'
        body = kwargs.copy()
        body.update({
            'name': name,
            'plugin_name': plugin_name,
            'hadoop_version': hadoop_version,
            'node_processes': node_processes,
            'flavor_id': flavor_id,
            'node_configs': node_configs or dict(),
        })
        return self._request_check_and_parse_resp(self.post, uri, 202,
                                                  body=json.dumps(body))

    def delete_node_group_template(self, tmpl_id):
        """Deletes the specified node group template by id."""

        uri = 'node-group-templates/%s' % tmpl_id
        return self._request_and_check_resp(self.delete, uri, 204)

    def list_plugins(self):
        """List all enabled plugins."""

        uri = 'plugins'
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def get_plugin(self, plugin_name, plugin_version=None):
        """Returns the details of a single plugin."""

        uri = 'plugins/%s' % plugin_name
        if plugin_version:
            uri += '/%s' % plugin_version
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def list_cluster_templates(self):
        """List all cluster templates for a user."""

        uri = 'cluster-templates'
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def get_cluster_template(self, tmpl_id):
        """Returns the details of a single cluster template."""

        uri = 'cluster-templates/%s' % tmpl_id
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def create_cluster_template(self, name, plugin_name, hadoop_version,
                                node_groups, cluster_configs=None,
                                **kwargs):
        """Creates cluster template with specified params.

        It supports passing additional params using kwargs and returns created
        object.
        """
        uri = 'cluster-templates'
        body = kwargs.copy()
        body.update({
            'name': name,
            'plugin_name': plugin_name,
            'hadoop_version': hadoop_version,
            'node_groups': node_groups,
            'cluster_configs': cluster_configs or dict(),
        })
        return self._request_check_and_parse_resp(self.post, uri, 202,
                                                  body=json.dumps(body))

    def delete_cluster_template(self, tmpl_id):
        """Deletes the specified cluster template by id."""

        uri = 'cluster-templates/%s' % tmpl_id
        return self._request_and_check_resp(self.delete, uri, 204)

    def update_cluster_template(self, tmpl_id, **kwargs):
        """Updates the specificed cluster template."""
        uri = 'cluster-templates/%s' % tmpl_id
        return self._request_check_and_parse_resp(self.put, uri, 202,
                                                  body=json.dumps(kwargs))

    def list_data_sources(self):
        """List all data sources for a user."""

        uri = 'data-sources'
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def get_data_source(self, source_id):
        """Returns the details of a single data source."""

        uri = 'data-sources/%s' % source_id
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def create_data_source(self, name, data_source_type, url, **kwargs):
        """Creates data source with specified params.

        It supports passing additional params using kwargs and returns created
        object.
        """
        uri = 'data-sources'
        body = kwargs.copy()
        body.update({
            'name': name,
            'type': data_source_type,
            'url': url
        })
        return self._request_check_and_parse_resp(self.post, uri,
                                                  202, body=json.dumps(body))

    def update_node_group_template(self, tmpl_id, **kwargs):
        """Updates the details of a single node group template."""
        uri = 'node-group-templates/%s' % tmpl_id
        return self._request_check_and_parse_resp(self.put, uri, 202,
                                                  body=json.dumps(kwargs))

    def delete_data_source(self, source_id):
        """Deletes the specified data source by id."""

        uri = 'data-sources/%s' % source_id
        return self._request_and_check_resp(self.delete, uri, 204)

    def update_data_source(self, source_id, **kwargs):
        """Updates a data source"""
        uri = 'data-sources/%s' % source_id
        return self._request_check_and_parse_resp(self.put, uri, 202,
                                                  body=json.dumps(kwargs))

    def list_job_binary_internals(self):
        """List all job binary internals for a user."""

        uri = 'job-binary-internals'
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def get_job_binary_internal(self, job_binary_id):
        """Returns the details of a single job binary internal."""

        uri = 'job-binary-internals/%s' % job_binary_id
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def create_job_binary_internal(self, name, data):
        """Creates job binary internal with specified params."""

        uri = 'job-binary-internals/%s' % name
        return self._request_check_and_parse_resp(self.put, uri, 202, data)

    def delete_job_binary_internal(self, job_binary_id):
        """Deletes the specified job binary internal by id."""

        uri = 'job-binary-internals/%s' % job_binary_id
        return self._request_and_check_resp(self.delete, uri, 204)

    def get_job_binary_internal_data(self, job_binary_id):
        """Returns data of a single job binary internal."""

        uri = 'job-binary-internals/%s/data' % job_binary_id
        return self._request_and_check_resp_data(self.get, uri, 200)

    def list_job_binaries(self):
        """List all job binaries for a user."""

        uri = 'job-binaries'
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def get_job_binary(self, job_binary_id):
        """Returns the details of a single job binary."""

        uri = 'job-binaries/%s' % job_binary_id
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def create_job_binary(self, name, url, extra=None, **kwargs):
        """Creates job binary with specified params.

        It supports passing additional params using kwargs and returns created
        object.
        """
        uri = 'job-binaries'
        body = kwargs.copy()
        body.update({
            'name': name,
            'url': url,
            'extra': extra or dict(),
        })
        return self._request_check_and_parse_resp(self.post, uri,
                                                  202, body=json.dumps(body))

    def delete_job_binary(self, job_binary_id):
        """Deletes the specified job binary by id."""

        uri = 'job-binaries/%s' % job_binary_id
        return self._request_and_check_resp(self.delete, uri, 204)

    def get_job_binary_data(self, job_binary_id):
        """Returns data of a single job binary."""

        uri = 'job-binaries/%s/data' % job_binary_id
        return self._request_and_check_resp_data(self.get, uri, 200)

    def list_jobs(self):
        """List all jobs for a user."""

        uri = 'jobs'
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def get_job(self, job_id):
        """Returns the details of a single job."""

        uri = 'jobs/%s' % job_id
        return self._request_check_and_parse_resp(self.get, uri, 200)

    def create_job(self, name, job_type, mains, libs=None, **kwargs):
        """Creates job with specified params.

        It supports passing additional params using kwargs and returns created
        object.
        """
        uri = 'jobs'
        body = kwargs.copy()
        body.update({
            'name': name,
            'type': job_type,
            'mains': mains,
            'libs': libs or list(),
        })
        return self._request_check_and_parse_resp(self.post, uri,
                                                  202, body=json.dumps(body))

    def delete_job(self, job_id):
        """Deletes the specified job by id."""

        uri = 'jobs/%s' % job_id
        return self._request_and_check_resp(self.delete, uri, 204)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sahara_tempest_plugin.tests.clients import base


class PluginsTest(base.BaseDataProcessingTest):

    def _check_plugins_list(self):
        plugins = self.client.plugins.list()
        plugins_names = [plugin.name for plugin in plugins]
        plugin = self.get_plugin()
        self.assertIn(plugin.name, plugins_names)

        return plugins_names

    def _check_plugins_get(self, plugins_names):
        for plugin_name in plugins_names:
            plugin = self.client.plugins.get(plugin_name)
            self.assertEqual(plugin_name, plugin.name)

            # check get_version_details
            for plugin_version in plugin.versions:
                detailed_plugin = self.client.plugins.get_version_details(
                    plugin_name, plugin_version)
                self.assertEqual(plugin_name, detailed_plugin.name)

                # check that required image tags contains name and version
                image_tags = detailed_plugin.required_image_tags
                self.assertIn(plugin_name, image_tags)
                self.assertIn(plugin_version, image_tags)

    def test_plugins(self):
        plugins_names = self._check_plugins_list()
        self._check_plugins_get(plugins_names)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import fixtures
import time

from tempest import config
from tempest.lib.cli import base
from tempest.test import BaseTestCase
from tempest.lib import exceptions as exc
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.common import plugin_utils

TEMPEST_CONF = config.CONF
DEL_RESULT = '''\
{} "{}" has been removed successfully.
'''
TEMPEST_ERROR_MESSAGE = 'No matches found.'


class ClientTestBase(base.ClientTestBase):
    """Base class for saharaclient tests.

    Establishes the sahara client and retrieves the essential environment
    information.
    """

    def _get_clients(self):
        cli_dir = os.environ.get('OS_SAHARA_TESTS_DIR', '')
        if not cli_dir:
            # if this is executed in a virtualenv, the command installed there
            # will be the first one.
            paths = os.environ.get('PATH').split(':')
            for path in paths:
                client_candidate = os.path.join(path, 'openstack')
                if os.path.isfile(client_candidate) and os.access(
                        client_candidate, os.X_OK):
                    cli_dir = path
                    break

        self.client_manager_admin = BaseTestCase.get_client_manager('admin')
        auth_provider = self.client_manager_admin.auth_provider
        self.project_network = BaseTestCase.get_tenant_network('admin')

        return base.CLIClient(
            username=auth_provider.credentials.get('username'),
            password=auth_provider.credentials.get('password'),
            tenant_name=auth_provider.credentials.get('tenant_name'),
            uri=auth_provider.base_url({'service': 'identity'}),
            cli_dir=cli_dir)

    def openstack(self, *args, **kwargs):
        return self.clients.openstack(*args, **kwargs)

    def listing_result(self, command):
        command_for_item = self.openstack('dataprocessing', params=command)
        result = self.parser.listing(command_for_item)
        return result

    def find_in_listing(self, result, value, field='name'):
        for line in result:
            if line['Field'].lower() == field.lower():
                self.assertEqual(line['Value'].lower(), value.lower())
                return
        raise self.skipException('No table to show information')

    def check_if_delete(self, command, name):
        delete_cmd = self.openstack('dataprocessing %s delete' % command,
                                    params=name)
        result = DEL_RESULT.format(command, name)
        # lower() is required because "command" in the result string could
        # have the first letter capitalized.
        self.assertEqual(delete_cmd.lower(), result.lower())

    def update_resource_value(self, command, value, params):
        new_value = data_utils.rand_name(value)
        command = '%s update %s' % (command, value)
        params = '%s %s' % (params, new_value)
        update_result = self.listing_result('%s %s' % (command, params))
        self.find_in_listing(update_result, new_value)
        return new_value

    def delete_resource(self, command, name):
        list_of_resources = self.listing_result('%s list' % command)
        list_of_resource_names = [r['Name'] for r in list_of_resources]
        if name in list_of_resource_names:
            self.openstack('dataprocessing %s delete' % command, params=name)

    def get_default_plugin(self):
        plugins = self.listing_result('plugin list')
        default_plugin_name = plugin_utils.get_default_plugin()
        for plugin in plugins:
            if plugin['Name'] == default_plugin_name:
                return plugin
        raise self.skipException('No available plugins for testing')

    def find_id_of_pool(self):
        floating_pool_list = self.openstack('network list --external')
        floating_pool = self.parser.listing(floating_pool_list)
        if not floating_pool:
            raise self.skipException('Floating pool ip list is empty')
        # if not empty, there should be at least one element
        return floating_pool[0]['ID']

    def _get_cluster_status(self, cluster_name):
        status = None
        show_cluster = self.listing_result('cluster show %s' % cluster_name)
        for line in show_cluster:
            if line['Field'] == 'Status':
                status = line['Value']
        if status is None:
            raise self.skipException('Can not find the cluster to get its '
                                     'status')
        return status

    def _get_resource_id(self, resource, resource_name):
        resource_id = None
        show_resource = self.listing_result('%s show %s'
                                            % (resource, resource_name))
        for line in show_resource:
            if line['Field'] == 'Id':
                resource_id = line['Value']
        if resource_id is None:
            raise self.skipException('No such %s exists' % resource)
        return resource_id

    def _poll_cluster_status(self, cluster_name):
        with fixtures.Timeout(TEMPEST_CONF.data_processing.cluster_timeout,
                              gentle=True):
            while True:
                status = self._get_cluster_status(cluster_name)
                if status == 'Active':
                    break
                if status == 'Error':
                    raise exc.TempestException("Cluster in %s state" % status)
                time.sleep(3)

    def wait_for_resource_deletion(self, name, type):
        # type can be cluster, cluster template or node group template string
        name_exist = False
        # if name exists in the command "type list" than tests should fail
        with fixtures.Timeout(300, gentle=True):
            while True:
                list_of_types = self.listing_result('%s list' % type)
                list_names = [p['Name'] for p in list_of_types]
                if name in list_names:
                    name_exist = True
                if not name_exist:
                    break

    def check_negative_scenarios(self, error_message, cmd, name):
        msg_exist = None
        try:
            self.openstack('dataprocessing %s' % cmd, params=name)
        except exc.CommandFailed as e:
            # lower() is required because "result" string could
            # have the first letter capitalized.
            output_msg = str(e).splitlines()
            for msg in output_msg:
                if msg.lower() == error_message.lower():
                    self.assertEqual(error_message.lower(), msg.lower())
                    msg_exist = True
            if not msg_exist:
                raise exc.TempestException('%s is not a part of output of '
                                           'executed command %s'
                                           % (error_message, cmd))
        else:
            raise exc.TempestException('%s %s in negative scenarios have been '
                                       'executed without any errors'
                                       % (cmd, name))

    @classmethod
    def tearDownClass(cls):
        if hasattr(super(ClientTestBase, cls), 'tearDownClass'):
            super(ClientTestBase, cls).tearDownClass()
        # this'll be needed as long as BaseTestCase functions
        # are used in this class, otherwise projects, users,
        # networks and routers created won't be deleted
        BaseTestCase.clear_credentials()



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import testtools
import yaml

from sahara_tests.scenario import validation


class TestValidation(testtools.TestCase):
    def test_validation(self):
        with open("sahara_tests/unit/scenario/vanilla2_7_1.yaml",
                  "r") as yaml_file:
            config = yaml.safe_load(yaml_file)
        self.assertIsNone(validation.validate(config))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from sahara_tempest_plugin.tests.cli import base

DELETE_RES = '''\
job "%s" deletion has been started.
'''


class SaharaJobCLITest(base.ClientTestBase):

    def openstack_job_list(self):
        self.assertTableStruct(self.listing_result('job list'), [
            'Id',
            'Cluster id',
            'Job id',
            'Status'
        ])

    def openstack_job_execute(self, cluster_name, job_template_name, input,
                              output):
        flag = ("%(cluster)s %(jt)s %(input)s %(output)s"
                % {'cluster': ' --cluster %s' % cluster_name,
                   'jt': ' --job-template %s' % job_template_name,
                   'input': ' --input %s' % input,
                   'output': ' --output %s' % output})
        result = self.listing_result('job execute %s' % flag)
        self.assertTableStruct(
            result,
            [
                'Field',
                'Value'
            ])
        for line in result:
            if line['Field'] == 'Id':
                return line['Value']

    def openstack_job_show(self, job_id):
        result = self.listing_result('job show %s' % job_id)
        check_table = False
        for line in result:
            if line['Field'] == 'Id':
                self.assertEqual(line['Value'], job_id)
                check_table = True
        if not check_table:
            raise self.skipException('No table to show information')

    def openstack_job_update(self, job_id):
        self.assertTableStruct(
            self.listing_result(
                'job update --public %s' % job_id), [
                'Field',
                'Value'
            ])

    def openstack_job_delete(self, job_id):
        delete_job = self.openstack('dataprocessing job delete',
                                    params=job_id)
        self.assertEqual(delete_job.lower(), DELETE_RES % job_id)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import time

from oslo_utils import timeutils
from keystoneauth1.identity import v3
from keystoneauth1 import session
from saharaclient.api import base as sab
from saharaclient import client as sahara_client
from tempest import config
from tempest.lib import exceptions
import tempest.test

from sahara_tempest_plugin.common import plugin_utils

TEMPEST_CONF = config.CONF

# cluster status
CLUSTER_STATUS_ACTIVE = "Active"
CLUSTER_STATUS_ERROR = "Error"


class ClusterErrorException(exceptions.TempestException):
    message = "Cluster failed to build and is in ERROR status"


class BaseDataProcessingTest(tempest.test.BaseTestCase):

    credentials = ('admin', 'primary')

    @classmethod
    def resource_setup(cls):
        cls.set_network_resources()
        super(BaseDataProcessingTest, cls).resource_setup()

        endpoint_type = TEMPEST_CONF.data_processing.endpoint_type
        catalog_type = TEMPEST_CONF.data_processing.catalog_type
        auth_url = TEMPEST_CONF.identity.uri

        credentials = cls.os_admin.credentials

        auth = v3.Password(auth_url=auth_url.replace('/v2.0', '/v3'),
                           username=credentials.username,
                           password=credentials.password,
                           project_name=credentials.tenant_name,
                           user_domain_name='default',
                           project_domain_name='default')

        ses = session.Session(auth=auth)

        cls.client = sahara_client.Client(
            TEMPEST_CONF.data_processing.saharaclient_version,
            session=ses,
            service_type=catalog_type,
            endpoint_type=endpoint_type)

        if TEMPEST_CONF.service_available.glance:
            # Check if glance v1 is available to determine which client to use.
            if TEMPEST_CONF.image_feature_enabled.api_v1:
                cls.image_client = cls.os_admin.image_client
            elif TEMPEST_CONF.image_feature_enabled.api_v2:
                cls.image_client = cls.os_admin.image_client_v2
            else:
                raise lib_exc.InvalidConfiguration(
                    'Either api_v1 or api_v2 must be True in '
                    '[image-feature-enabled].')
        cls.object_client = cls.os_primary.object_client
        cls.container_client = cls.os_primary.container_client
        cls.networks_client = cls.os_primary.compute_networks_client

        if TEMPEST_CONF.network.floating_network_name:
            cls.floating_ip_pool = TEMPEST_CONF.network.floating_network_name
            if TEMPEST_CONF.service_available.neutron:
                cls.floating_ip_pool = \
                    cls.get_floating_ip_pool_id_for_neutron()
        else:
            cls.floating_ip_pool = TEMPEST_CONF.network.public_network_id

        test_image_name = TEMPEST_CONF.data_processing.test_image_name

        cls.test_image_id = cls.get_image_id(test_image_name)

        default_plugin = cls.get_plugin()
        plugin_dict = default_plugin.to_dict()
        default_version = plugin_utils.get_default_version(plugin_dict)

        cls.worker_template = (
            plugin_utils.get_node_group_template('worker1',
                                                 default_version,
                                                 cls.floating_ip_pool))

        cls.master_template = (
            plugin_utils.get_node_group_template('master1',
                                                 default_version,
                                                 cls.floating_ip_pool))

        cls.cluster_template = (
            plugin_utils.get_cluster_template(
                default_version=default_version))

        cls.swift_data_source_with_creds = {
            'url': 'swift://sahara-container/input-source',
            'description': 'Test data source',
            'type': 'swift',
            'credentials': {
                'user': 'test',
                'password': '123'
            }
        }

        cls.local_hdfs_data_source = {
            'url': 'input-source',
            'description': 'Test data source',
            'type': 'hdfs',
        }

        cls.external_hdfs_data_source = {
            'url': 'hdfs://test-master-node/usr/hadoop/input-source',
            'description': 'Test data source',
            'type': 'hdfs'
        }

    @classmethod
    def get_floating_ip_pool_id_for_neutron(cls):
        net_id = cls._find_network_by_name(
            TEMPEST_CONF.network.floating_network_name)
        if not net_id:
            raise exceptions.NotFound(
                'Floating IP pool \'%s\' not found in pool list.'
                % TEMPEST_CONF.network.floating_network_name)
        return net_id

    @classmethod
    def get_private_network_id(cls):
        net_id = cls._find_network_by_name(
            TEMPEST_CONF.compute.fixed_network_name)
        if not net_id:
            raise exceptions.NotFound(
                'Private network \'%s\' not found in network list.'
                % TEMPEST_CONF.compute.fixed_network_name)
        return net_id

    @classmethod
    def _find_network_by_name(cls, network_name):
        for network in cls.networks_client.list_networks()['networks']:
            if network['label'] == network_name:
                return network['id']
        return None

    @classmethod
    def get_image_id(cls, image_name):
        for image in cls.image_client.list_images()['images']:
            if image['name'] == image_name:
                return image['id']
        raise exceptions.NotFound('Image \'%s\' not found in the image list.'
                                  % (image_name))

    @classmethod
    def get_plugin(cls):
        plugins = cls.client.plugins.list()
        plugin_name = plugin_utils.get_default_plugin()
        for plugin in plugins:
            if plugin.name == plugin_name:
                return plugin
        raise exceptions.NotFound('No available plugins for testing')

    def create_node_group_template(self, name, **kwargs):

        resp_body = self.client.node_group_templates.create(
            name, **kwargs)

        self.addCleanup(self.delete_resource,
                        self.client.node_group_templates, resp_body.id)

        return resp_body

    def create_cluster_template(self, name, **kwargs):

        resp_body = self.client.cluster_templates.create(
            name, **kwargs)

        self.addCleanup(self.delete_resource,
                        self.client.cluster_templates, resp_body.id)

        return resp_body

    def create_data_source(self, name, url, description, type,
                           credentials=None):

        user = credentials['user'] if credentials else None
        pas = credentials['password'] if credentials else None

        resp_body = self.client.data_sources.create(
            name, description, type, url, credential_user=user,
            credential_pass=pas)

        self.addCleanup(self.delete_resource,
                        self.client.data_sources, resp_body.id)

        return resp_body

    def create_job_binary(self, name, url, description, extra=None):

        resp_body = self.client.job_binaries.create(
            name, url, description, extra)

        self.addCleanup(self.delete_resource,
                        self.client.job_binaries, resp_body.id)

        return resp_body

    def create_job_binary_internal(self, name, data):

        resp_body = self.client.job_binary_internals.create(name, data)

        self.addCleanup(self.delete_resource,
                        self.client.job_binary_internals, resp_body.id)

        return resp_body

    def create_job(self, name, job_type, mains, libs=None, description=None):

        libs = libs or ()
        description = description or ''

        resp_body = self.client.jobs.create(
            name, job_type, mains, libs, description)

        self.addCleanup(self.delete_resource, self.client.jobs, resp_body.id)

        return resp_body

    def create_cluster(self, name, **kwargs):

        resp_body = self.client.clusters.create(name, **kwargs)

        self.addCleanup(self.delete_resource, self.client.clusters,
                        resp_body.id)

        return resp_body

    def check_cluster_active(self, cluster_id):
        timeout = TEMPEST_CONF.data_processing.cluster_timeout
        s_time = timeutils.utcnow()
        while timeutils.delta_seconds(s_time, timeutils.utcnow()) < timeout:
            cluster = self.client.clusters.get(cluster_id)
            if cluster.status == CLUSTER_STATUS_ACTIVE:
                return
            if cluster.status == CLUSTER_STATUS_ERROR:
                raise ClusterErrorException(
                    'Cluster failed to build and is in %s status.' %
                    CLUSTER_STATUS_ERROR)
            time.sleep(TEMPEST_CONF.data_processing.request_timeout)
        raise exceptions.TimeoutException(
            'Cluster failed to get to %s status within %d seconds.'
            % (CLUSTER_STATUS_ACTIVE, timeout))

    def create_job_execution(self, **kwargs):

        resp_body = self.client.job_executions.create(**kwargs)

        self.addCleanup(self.delete_resource, self.client.job_executions,
                        resp_body.id)

        return resp_body

    def create_container(self, name):

        self.container_client.create_container(name)

        self.addCleanup(self.delete_swift_container, name)

    def delete_resource(self, resource_client, resource_id):
        try:
            resource_client.delete(resource_id)
        except sab.APIException:
            pass
        else:
            self.delete_timeout(resource_client, resource_id)

    def delete_timeout(
            self, resource_client, resource_id,
            timeout=TEMPEST_CONF.data_processing.cluster_timeout):

        start = timeutils.utcnow()
        while timeutils.delta_seconds(start, timeutils.utcnow()) < timeout:
            try:
                resource_client.get(resource_id)
            except sab.APIException as sahara_api_exception:
                if 'not found' in str(sahara_api_exception):
                    return
                raise sahara_api_exception

            time.sleep(TEMPEST_CONF.data_processing.request_timeout)

        raise exceptions.TimeoutException(
            'Failed to delete resource "%s" in %d seconds.'
            % (resource_id, timeout))

    def delete_swift_container(self, container):
        objects = ([obj['name'] for obj in
                    self.container_client.list_all_container_objects(
                        container)])
        for obj in objects:
            self.object_client.delete_object(container, obj)
        self.container_client.delete_container(container)



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from testtools import testcase as tc

from tempest.lib import decorators
from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.api import base as dp_base


class DataSourceTest(dp_base.BaseDataProcessingTest):
    @classmethod
    def resource_setup(cls):
        super(DataSourceTest, cls).resource_setup()
        cls.swift_data_source_with_creds = {
            'url': 'swift://sahara-container.sahara/input-source',
            'description': 'Test data source',
            'credentials': {
                'user': cls.os_primary.credentials.username,
                'password': cls.os_primary.credentials.password
            },
            'type': 'swift'
        }
        cls.swift_data_source = cls.swift_data_source_with_creds.copy()
        del cls.swift_data_source['credentials']

        cls.local_hdfs_data_source = {
            'url': 'input-source',
            'description': 'Test data source',
            'type': 'hdfs'
        }

        cls.external_hdfs_data_source = {
            'url': 'hdfs://172.18.168.2:8020/usr/hadoop/input-source',
            'description': 'Test data source',
            'type': 'hdfs'
        }

    def _create_data_source(self, source_body, source_name=None):
        """Creates Data Source with optional name specified.

        It creates a link to input-source file (it may not exist), ensures
        source name and response body. Returns id and name of created source.
        """
        if not source_name:
            # generate random name if it's not specified
            source_name = data_utils.rand_name('sahara-data-source')

        # create data source
        resp_body = self.create_data_source(source_name, **source_body)

        # ensure that source created successfully
        self.assertEqual(source_name, resp_body['name'])
        if source_body['type'] == 'swift':
            source_body = self.swift_data_source
        self.assertDictContainsSubset(source_body, resp_body)

        return resp_body['id'], source_name

    def _list_data_sources(self, source_info):
        # check for data source in list
        sources = self.client.list_data_sources()['data_sources']
        sources_info = [(source['id'], source['name']) for source in sources]
        self.assertIn(source_info, sources_info)

    def _get_data_source(self, source_id, source_name, source_body):
        # check data source fetch by id
        source = self.client.get_data_source(source_id)['data_source']
        self.assertEqual(source_name, source['name'])
        self.assertDictContainsSubset(source_body, source)

    def _delete_data_source(self, source_id):
        # delete the data source by id
        self.client.delete_data_source(source_id)
        self.wait_for_resource_deletion(source_id, self.client.get_data_source)

        # assert data source does not exist anymore
        sources = self.client.list_data_sources()['data_sources']
        sources_ids = [source['id'] for source in sources]
        self.assertNotIn(source_id, sources_ids)

    def _update_data_source(self, source_id):
        new_source_name = data_utils.rand_name('sahara-data-source')
        body = {'name': new_source_name}
        updated_source = self.client.update_data_source(source_id, **body)
        source = updated_source['data_source']
        self.assertEqual(new_source_name, source['name'])

    @tc.attr('smoke')
    @decorators.idempotent_id('9e0e836d-c372-4fca-91b7-b66c3e9646c8')
    def test_swift_data_source_create(self):
        self._create_data_source(self.swift_data_source_with_creds)

    @tc.attr('smoke')
    @decorators.idempotent_id('3cb87a4a-0534-4b97-9edc-8bbc822b68a0')
    def test_swift_data_source_list(self):
        source_info = (
            self._create_data_source(self.swift_data_source_with_creds))
        self._list_data_sources(source_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('fc07409b-6477-4cb3-9168-e633c46b227f')
    def test_swift_data_source_get(self):
        source_id, source_name = (
            self._create_data_source(self.swift_data_source_with_creds))
        self._get_data_source(source_id, source_name, self.swift_data_source)

    @tc.attr('smoke')
    @decorators.idempotent_id('df53669c-0cd1-4cf7-b408-4cf215d8beb8')
    def test_swift_data_source_delete(self):
        source_id, _ = (
            self._create_data_source(self.swift_data_source_with_creds))
        self._delete_data_source(source_id)

    @tc.attr('smoke')
    @decorators.idempotent_id('44398efb-c2a8-4a20-97cd-509c49b5d25a')
    def test_swift_data_source_update(self):
        source_id, _ = (
            self._create_data_source(self.swift_data_source_with_creds))
        self._update_data_source(source_id)

    @tc.attr('smoke')
    @decorators.idempotent_id('88505d52-db01-4229-8f1d-a1137da5fe2d')
    def test_local_hdfs_data_source_create(self):
        self._create_data_source(self.local_hdfs_data_source)

    @tc.attr('smoke')
    @decorators.idempotent_id('81d7d42a-d7f6-4d9b-b38c-0801a4dfe3c2')
    def test_local_hdfs_data_source_list(self):
        source_info = self._create_data_source(self.local_hdfs_data_source)
        self._list_data_sources(source_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('ec0144c6-db1e-4169-bb06-7abae14a8443')
    def test_local_hdfs_data_source_get(self):
        source_id, source_name = (
            self._create_data_source(self.local_hdfs_data_source))
        self._get_data_source(
            source_id, source_name, self.local_hdfs_data_source)

    @tc.attr('smoke')
    @decorators.idempotent_id('e398308b-4230-4f86-ba10-9b0b60a59c8d')
    def test_local_hdfs_data_source_delete(self):
        source_id, _ = self._create_data_source(self.local_hdfs_data_source)
        self._delete_data_source(source_id)

    @tc.attr('smoke')
    @decorators.idempotent_id('16a71f3b-0095-431c-b542-c871e1f95e1f')
    def test_local_hdfs_data_source_update(self):
        source_id, _ = self._create_data_source(self.local_hdfs_data_source)
        self._update_data_source(source_id)

    @tc.attr('smoke')
    @decorators.idempotent_id('bfd91128-e642-4d95-a973-3e536962180c')
    def test_external_hdfs_data_source_create(self):
        self._create_data_source(self.external_hdfs_data_source)

    @tc.attr('smoke')
    @decorators.idempotent_id('92e2be72-f7ab-499d-ae01-fb9943c90d8e')
    def test_external_hdfs_data_source_list(self):
        source_info = self._create_data_source(self.external_hdfs_data_source)
        self._list_data_sources(source_info)

    @tc.attr('smoke')
    @decorators.idempotent_id('a31edb1b-6bc6-4f42-871f-70cd243184ac')
    def test_external_hdfs_data_source_get(self):
        source_id, source_name = (
            self._create_data_source(self.external_hdfs_data_source))
        self._get_data_source(
            source_id, source_name, self.external_hdfs_data_source)

    @tc.attr('smoke')
    @decorators.idempotent_id('295924cd-a085-4b45-aea8-0707cdb2da7e')
    def test_external_hdfs_data_source_delete(self):
        source_id, _ = self._create_data_source(self.external_hdfs_data_source)
        self._delete_data_source(source_id)

    @tc.attr('smoke')
    @decorators.idempotent_id('9b317861-95db-44bc-9b4b-80d23feade3f')
    def test_external_hdfs_data_source_update(self):
        source_id, _ = self._create_data_source(self.external_hdfs_data_source)
        self._update_data_source(source_id)



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import mock
from saharaclient.api import cluster_templates
from saharaclient.api import clusters
from saharaclient.api import data_sources
from saharaclient.api import images
from saharaclient.api import job_binaries
from saharaclient.api import job_binary_internals
from saharaclient.api import job_executions
from saharaclient.api import jobs
from saharaclient.api import node_group_templates
from saharaclient.api import plugins
from tempest.lib import exceptions as exc
import testtools

from sahara_tests.scenario import base
from sahara_tests.scenario import timeouts


class FakeSaharaClient(object):
    def __init__(self):
        self.clusters = clusters.ClusterManager(None)
        self.cluster_templates = cluster_templates.ClusterTemplateManager(None)
        self.node_group_templates = (node_group_templates.
                                     NodeGroupTemplateManager(None))
        self.plugins = plugins.PluginManager(None)
        self.images = images.ImageManager(None)
        self.data_sources = data_sources.DataSourceManager(None)
        self.jobs = jobs.JobsManager(None)
        self.job_executions = job_executions.JobExecutionsManager(None)
        self.job_binaries = job_binaries.JobBinariesManager(None)
        self.job_binary_internals = (
            job_binary_internals.JobBinaryInternalsManager(None))


class FakeCluster(object):
    def __init__(self, is_transient=False, provision_progress=[], ng=[]):
        self.is_transient = is_transient
        self.provision_progress = provision_progress
        self.node_groups = ng


class FakeResponse(object):
    def __init__(self, set_id=None, set_status=None, status_description=None,
                 node_groups=None, url=None, job_id=None, name=None,
                 job_type=None, verification=None):
        self.id = set_id
        self.status = set_status
        self.status_description = status_description
        self.node_groups = node_groups
        self.url = url
        self.job_id = job_id
        self.name = name
        self.type = job_type
        self.verification = verification


class FakeFlavor(object):
    def __init__(self, flavor_id=None, name=None):
        self.id = flavor_id
        self.name = name


class TestBase(testtools.TestCase):
    def setUp(self):
        super(TestBase, self).setUp()
        with mock.patch(
                'sahara_tests.scenario.base.BaseTestCase.__init__'
        ) as mock_init:
            mock_init.return_value = None
            self.base_scenario = base.BaseTestCase()
        self.base_scenario.credentials = {'os_username': 'admin',
                                          'os_password': 'nova',
                                          'os_tenant': 'admin',
                                          'os_auth_url':
                                              'http://localhost:5000/v2.0',
                                          'sahara_service_type':
                                              'data-processing-local',
                                          'sahara_url':
                                              'http://sahara_host:8386/v1.1',
                                          'ssl_cert': 'sahara_tests/unit/'
                                                      'scenario/dummy.crt',
                                          'ssl_verify': True}
        self.base_scenario.plugin_opts = {'plugin_name': 'vanilla',
                                          'hadoop_version': '2.7.1'}
        self.base_scenario.network = {'type': 'neutron',
                                      'private_network': 'changed_private',
                                      'public_network': 'changed_public',
                                      'auto_assignment_floating_ip': False}
        self.base_scenario.testcase = {
            'node_group_templates': [
                {
                    'name': 'master',
                    'node_processes': ['namenode', 'oozie', 'resourcemanager'],
                    'flavor': '2',
                    'is_proxy_gateway': True
                },
                {
                    'name': 'worker',
                    'node_processes': ['datanode', 'nodemanager'],
                    'flavor': '2'
                }],
            'cluster_template': {
                'name': 'test_name_ct',
                'node_group_templates': {
                    'master': 1,
                    'worker': 3
                }
            },
            'timeout_poll_cluster_status': 300,
            'timeout_delete_resource': 300,
            'timeout_poll_jobs_status': 2,
            'timeout_check_transient': 3,
            'retain_resources': True,
            'image': 'image_name',
            'edp_batching': 1,
            "edp_jobs_flow": {
                "test_flow": [
                    {
                        "type": "Pig",
                        "input_datasource": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/"
                                      "top-todoers/data/input"
                        },
                        "output_datasource": {
                            "type": "hdfs",
                            "destination": "/user/hadoop/edp-output"
                        },
                        "main_lib": {
                            "type": "swift",
                            "source": "sahara_tests/scenario/defaults/"
                                      "edp-examples/edp-pig/"
                                      "top-todoers/example.pig"
                        }
                    }
                ]
            }
        }
        self.base_scenario.ng_id_map = {'worker': 'set_id', 'master': 'set_id'}
        self.base_scenario.ng_name_map = {}
        self.base_scenario.key_name = 'test_key'
        self.base_scenario.key = 'key_from_yaml'
        self.base_scenario.template_path = ('sahara_tests/scenario/templates/'
                                            'vanilla/2.7.1')
        self.job = self.base_scenario.testcase["edp_jobs_flow"].get(
            'test_flow')[0]
        self.base_scenario.cluster_id = 'some_id'
        self.base_scenario.proxy_ng_name = False
        self.base_scenario.proxy = False
        self.base_scenario.setUpClass()
        timeouts.Defaults.init_defaults(self.base_scenario.testcase)

    @mock.patch('keystoneauth1.identity.v3.Password')
    @mock.patch('keystoneauth1.session.Session')
    @mock.patch('glanceclient.client.Client', return_value=None)
    @mock.patch('saharaclient.client.Client', return_value=None)
    @mock.patch('novaclient.client.Client', return_value=None)
    @mock.patch('neutronclient.neutron.client.Client', return_value=None)
    @mock.patch('swiftclient.client.Connection', return_value=None)
    def test__init_clients(self, swift, neutron, nova, sahara, glance,
                           m_session, m_auth):
        fake_session = mock.Mock()
        fake_auth = mock.Mock()
        m_session.return_value = fake_session
        m_auth.return_value = fake_auth

        self.base_scenario._init_clients()

        sahara.assert_called_with('1.1',
                                  session=fake_session,
                                  service_type='data-processing-local',
                                  sahara_url='http://sahara_host:8386/v1.1')
        swift.assert_called_with(
            auth_version='2.0', user='admin', key='nova', insecure=False,
            cacert='sahara_tests/unit/scenario/dummy.crt',
            tenant_name='admin', authurl='http://localhost:5000/v2.0')

        nova.assert_called_with('2', session=fake_session)
        neutron.assert_called_with('2.0', session=fake_session)
        glance.assert_called_with('2', session=fake_session)

        m_auth.assert_called_with(auth_url='http://localhost:5000/v3',
                                  username='admin',
                                  password='nova',
                                  project_name='admin',
                                  user_domain_name='default',
                                  project_domain_name='default')
        m_session.assert_called_with(
            auth=fake_auth,
            cert='sahara_tests/unit/scenario/dummy.crt', verify=True)

    @mock.patch('neutronclient.v2_0.client.Client.list_networks',
                return_value={'networks': [{'id': '2314'}]})
    @mock.patch('saharaclient.api.node_group_templates.'
                'NodeGroupTemplateManager.create',
                return_value=FakeResponse(set_id='id_ng'))
    def test__create_node_group_template(self, mock_del, mock_saharaclient):
        self.base_scenario._init_clients()
        self.assertEqual({'worker': 'id_ng', 'master': 'id_ng'},
                         self.base_scenario._create_node_group_templates())

    @mock.patch('saharaclient.api.node_group_templates.'
                'NodeGroupTemplateManager.create',
                return_value=FakeResponse(set_id='id_ng'))
    @mock.patch('neutronclient.v2_0.client.Client.list_networks',
                return_value={'networks': [
                    {'id': '342'}
                ]})
    @mock.patch('neutronclient.v2_0.client.Client.create_security_group',
                return_value={'security_group': {'id': '213'}})
    @mock.patch('sahara_tests.scenario.clients.NeutronClient'
                '.add_security_group_rule_for_neutron',
                return_value='sg_name')
    @mock.patch('sahara_tests.scenario.clients.NeutronClient'
                '.delete_security_group_for_neutron',
                return_value=None)
    def test__create_security_group_uuid(self, mock_del, mock_add_rule,
                                         mock_sg, mock_neutron, mock_ng):
        self.base_scenario.network['public_network'] = (
            '692dcc5b-1205-4645-8a12-2558579ed17e')
        self.base_scenario._init_clients()
        for ng in self.base_scenario.testcase['node_group_templates']:
            ng['auto_security_group'] = False
        self.assertEqual({'master': 'id_ng', 'worker': 'id_ng'},
                         self.base_scenario._create_node_group_templates())

    @mock.patch('saharaclient.api.node_group_templates.'
                'NodeGroupTemplateManager.create',
                return_value=FakeResponse(set_id='id_ng'))
    @mock.patch('neutronclient.v2_0.client.Client.list_networks',
                return_value={'networks': [
                    {'id': '342'}
                ]})
    @mock.patch('neutronclient.v2_0.client.Client.create_security_group',
                return_value={'security_group': {'id': '213'}})
    @mock.patch('sahara_tests.scenario.clients.NeutronClient'
                '.create_security_group_for_neutron',
                return_value='sg_name')
    @mock.patch('neutronclient.v2_0.client.Client.create_security_group_rule',
                return_value=None)
    @mock.patch('neutronclient.v2_0.client.Client.delete_security_group',
                return_value=None)
    def test__create_security_group(self, mock_del, mock_create, mock_sg,
                                    mock_sgn, mock_list, mock_ng):
        self.base_scenario._init_clients()
        for ng in self.base_scenario.testcase['node_group_templates']:
            ng['auto_security_group'] = False
        self.assertEqual({'master': 'id_ng', 'worker': 'id_ng'},
                         self.base_scenario._create_node_group_templates())

    @mock.patch('sahara_tests.scenario.clients.NeutronClient.get_network_id',
                return_value='mock_net')
    @mock.patch('saharaclient.api.cluster_templates.'
                'ClusterTemplateManager.create',
                return_value=FakeResponse(set_id='id_ct'))
    def test__create_cluster_template(self, mock_ct, mock_neutron):
        self.base_scenario._init_clients()
        self.assertEqual('id_ct',
                         self.base_scenario._create_cluster_template())

    @mock.patch('saharaclient.api.images.ImageManager.get',
                return_value=FakeResponse(set_id='image'))
    @mock.patch('sahara_tests.scenario.clients.GlanceClient.get_image_id',
                return_value='mock_image')
    @mock.patch('saharaclient.api.clusters.ClusterManager.create',
                return_value=FakeResponse(set_id='id_cluster'))
    def test__create_cluster(self, mock_cluster_manager, mock_glance,
                             mock_image):
        self.base_scenario._init_clients()
        self.assertEqual('id_cluster',
                         self.base_scenario._create_cluster('id_ct'))

    @mock.patch('sahara_tests.scenario.clients.NeutronClient.get_network_id',
                return_value='mock_net')
    @mock.patch('saharaclient.api.base.ResourceManager._get',
                return_value=FakeResponse(
                    set_status=base.CLUSTER_STATUS_ACTIVE))
    def test__poll_cluster_status(self, mock_status, mock_neutron):
        self.base_scenario._init_clients()
        self.assertIsNone(
            self.base_scenario._poll_cluster_status('id_cluster'))

    @mock.patch('saharaclient.api.base.ResourceManager._get')
    def test_check_event_log_feature(self, mock_resp):
        self.base_scenario._init_clients()

        self.assertIsNone(self.base_scenario._check_event_logs(
            FakeCluster(True, [])))
        self.assertIsNone(self.base_scenario._check_event_logs(
            FakeCluster(False, [{'successful': True}])))

        with testtools.ExpectedException(exc.TempestException):
            self.base_scenario._check_event_logs(
                FakeCluster(False, [{'successful': False}]))

        with testtools.ExpectedException(exc.TempestException):
            self.base_scenario._check_event_logs(
                FakeCluster(False, [{'successful': None}]))

    @mock.patch('saharaclient.api.base.ResourceManager._update',
                return_value=FakeResponse(set_id='id_internal_db_data'))
    def test__create_internal_db_data(self, mock_update):
        self.base_scenario._init_clients()
        self.assertEqual('internal-db://id_internal_db_data',
                         self.base_scenario._create_internal_db_data(
                             'sahara_tests/unit/scenario/vanilla2_7_1.yaml'))

    @mock.patch('swiftclient.client.Connection.put_container',
                return_value=None)
    def test__create_swift_data(self, mock_swiftclient):
        self.base_scenario._init_clients()
        self.assertIn('swift://sahara-tests-',
                      self.base_scenario._create_swift_data())

    @mock.patch('swiftclient.client.Connection.put_container',
                return_value=None)
    def test__get_swift_container(self, mock_swiftclient):
        self.base_scenario._init_clients()
        self.assertIn('sahara-tests-',
                      self.base_scenario._get_swift_container())

    @mock.patch('saharaclient.api.base.ResourceManager._create',
                return_value=FakeResponse(set_id='id_for_datasource'))
    @mock.patch('swiftclient.client.Connection.put_container',
                return_value=None)
    @mock.patch('swiftclient.client.Connection.put_object',
                return_value=None)
    def test__create_datasources(self, mock_swiftcontainer, mock_swiftobject,
                                 mock_create):
        self.base_scenario._init_clients()
        self.assertEqual(('id_for_datasource', 'id_for_datasource'),
                         self.base_scenario._create_datasources(
                             self.job))

    @mock.patch('saharaclient.api.base.ResourceManager._create',
                return_value=FakeResponse(set_id='id_for_job_binaries'))
    @mock.patch('swiftclient.client.Connection.put_object',
                return_value=None)
    @mock.patch('swiftclient.client.Connection.put_container',
                return_value=None)
    def test__create_create_job_binaries(self, mock_swiftcontainer,
                                         mock_swiftobject,
                                         mock_sahara_create):
        self.base_scenario._init_clients()
        self.assertEqual((['id_for_job_binaries'], []),
                         self.base_scenario._create_job_binaries(
                             self.job))

    @mock.patch('saharaclient.api.base.ResourceManager._create',
                return_value=FakeResponse(set_id='id_for_job_binary'))
    @mock.patch('swiftclient.client.Connection.put_object',
                return_value=None)
    @mock.patch('swiftclient.client.Connection.put_container',
                return_value=None)
    @mock.patch('saharaclient.client.Client', return_value=FakeSaharaClient())
    def test__create_create_job_binary(self, mock_saharaclient,
                                       mock_swiftcontainer, mock_swiftobject,
                                       mock_sahara_create):
        self.base_scenario._init_clients()
        self.assertEqual('id_for_job_binary',
                         self.base_scenario._create_job_binary(self.job.get(
                             'input_datasource')))

    @mock.patch('saharaclient.api.base.ResourceManager._create',
                return_value=FakeResponse(set_id='id_for_job'))
    def test__create_job(self, mock_client):
        self.base_scenario._init_clients()
        self.assertEqual('id_for_job',
                         self.base_scenario._create_job(
                             'Pig',
                             ['id_for_job_binaries'],
                             []))

    @mock.patch('sahara_tests.scenario.clients.SaharaClient.get_cluster_id',
                return_value='cluster_id')
    @mock.patch('sahara_tests.scenario.clients.SaharaClient.get_cluster',
                return_value=FakeCluster(ng=[]))
    @mock.patch('sahara_tests.scenario.base.BaseTestCase.check_cinder',
                return_value=None)
    @mock.patch('sahara_tests.scenario.clients.SaharaClient.get_job_status',
                return_value='KILLED')
    @mock.patch('saharaclient.api.base.ResourceManager._get',
                return_value=FakeResponse(set_id='id_for_run_job_get',
                                          job_type='Java',
                                          name='test_job'))
    @mock.patch('saharaclient.api.base.ResourceManager._create',
                return_value=FakeResponse(set_id='id_for_run_job_create'))
    @mock.patch('sahara_tests.scenario.base.BaseTestCase.'
                '_poll_cluster_status',
                return_value=None)
    @mock.patch('sahara_tests.scenario.base.BaseTestCase.'
                '_create_node_group_templates',
                return_value='id_node_group_template')
    @mock.patch('sahara_tests.scenario.base.BaseTestCase.'
                '_create_cluster_template',
                return_value='id_cluster_template')
    @mock.patch('sahara_tests.scenario.base.BaseTestCase._create_cluster',
                return_value='id_cluster')
    @mock.patch('sahara_tests.scenario.base.BaseTestCase._create_job',
                return_value='id_for_job')
    @mock.patch('sahara_tests.scenario.base.BaseTestCase._create_job_binaries',
                return_value=(['id_for_job_binaries'], []))
    @mock.patch('sahara_tests.scenario.base.BaseTestCase._create_datasources',
                return_value=('id_for_datasource', 'id_for_datasource'))
    @mock.patch('sahara_tests.scenario.base.BaseTestCase.check_verification')
    def test_check_run_jobs(self, mock_verification, mock_datasources,
                            mock_job_binaries, mock_job,
                            mock_node_group_template, mock_cluster_template,
                            mock_cluster, mock_cluster_status, mock_create,
                            mock_get, mock_client, mock_cinder, mock_get_cl,
                            mock_get_cluster_id):
        self.base_scenario._init_clients()
        self.base_scenario.create_cluster()
        self.base_scenario.testcase["edp_jobs_flow"] = [
            {
                "type": "Pig",
                "input_datasource": {
                    "type": "swift",
                    "source": "sahara_tests/scenario/defaults/edp-examples/"
                              "edp-pig/top-todoers/"
                              "data/input"
                },
                "output_datasource": {
                    "type": "hdfs",
                    "destination": "/user/hadoop/edp-output"
                },
                "main_lib": {
                    "type": "swift",
                    "source": "sahara_tests/scenario/defaults/edp-examples/"
                              "edp-pig/top-todoers/"
                              "example.pig"
                }
            }
        ]
        with mock.patch('time.sleep'):
            self.assertIsNone(self.base_scenario.check_run_jobs())
        self.assertIn("Job with id=id_for_run_job_create, name=test_job, "
                      "type=Java has status KILLED",
                      self.base_scenario._results[-1]['traceback'][-1])

    @mock.patch('sahara_tests.scenario.base.BaseTestCase._poll_cluster_status',
                return_value=None)
    @mock.patch('saharaclient.api.base.ResourceManager._get',
                return_value=FakeResponse(set_id='id_scale_get'))
    @mock.patch('saharaclient.api.base.ResourceManager._update',
                return_value=FakeResponse(set_id='id_scale_update'))
    def test_check_scale(self, mock_update, mock_get, mock_poll):
        self.base_scenario._init_clients()
        self.base_scenario.ng_id_map = {'vanilla-worker': 'set_id-w',
                                        'vanilla-master': 'set_id-m'}
        self.base_scenario.ng_name_map = {'vanilla-worker': 'worker-123',
                                          'vanilla-master': 'master-321'}
        self.base_scenario.cluster_id = 'cluster_id'
        self.assertIsNone(self.base_scenario.check_scale())

    @mock.patch('sahara_tests.scenario.clients.NeutronClient.get_network_id',
                return_value='mock_net')
    @mock.patch('saharaclient.api.base.ResourceManager._get',
                return_value=FakeResponse(set_status='Error',
                                          status_description=""))
    def test_errormsg(self, mock_status, mock_neutron):
        self.base_scenario._init_clients()
        with testtools.ExpectedException(exc.TempestException):
            self.base_scenario._poll_cluster_status('id_cluster')

    def test_get_nodes_with_process(self):
        self.base_scenario._init_clients()
        with mock.patch(
                'sahara_tests.scenario.clients.SaharaClient.get_cluster',
                return_value=FakeResponse(node_groups=[
                    {
                        'node_processes': ['test'],
                        'instances': ['test_instance']
                    }
                ])):
            self.assertEqual(
                ['test_instance'],
                self.base_scenario._get_nodes_with_process('test')
            )

        with mock.patch(
                'sahara_tests.scenario.clients.SaharaClient.get_cluster',
                return_value=FakeResponse(node_groups=[
                    {
                        'node_processes': 'test',
                        'instances': []
                    }
                ])):
            self.assertEqual(
                [], self.base_scenario._get_nodes_with_process('test'))

    def test_get_node_list_with_volumes(self):
        self.base_scenario._init_clients()
        with mock.patch(
                'sahara_tests.scenario.clients.SaharaClient.get_cluster',
                return_value=FakeResponse(node_groups=[
                    {
                        'node_processes': 'test',
                        'volumes_per_node': 2,
                        'volume_mount_prefix': 2,
                        'instances': [
                            {
                                'management_ip': 'test_ip'
                            }
                        ]
                    }
                ])):
            self.assertEqual(
                [{
                    'node_ip': 'test_ip',
                    'volume_count': 2,
                    'volume_mount_prefix': 2
                }], self.base_scenario._get_node_list_with_volumes())

    @mock.patch('sahara_tests.scenario.clients.SaharaClient.get_datasource')
    def test_put_io_data_to_configs(self, get_datasources):
        self.base_scenario._init_clients()
        get_datasources.side_effect = [
            mock.Mock(id='1', url="swift://cont/input"),
            mock.Mock(id='2', url="hdfs://cont/output")
        ]
        configs = {'args': ['2', "{input_datasource}",
                            "{output_datasource}"]}
        self.assertEqual({'args': ['2', 'swift://cont/input',
                                   'hdfs://cont/output']},
                         self.base_scenario._put_io_data_to_configs(
            configs, '1', '2'))

    @mock.patch('sahara_tests.scenario.base.BaseTestCase.addCleanup')
    @mock.patch('novaclient.v2.flavors.FlavorManager.create',
                return_value=FakeFlavor(flavor_id='created_flavor_id'))
    def test_get_flavor_id_anonymous(self, mock_create_flavor, mock_base):
        self.base_scenario._init_clients()
        self.assertEqual('created_flavor_id',
                         self.base_scenario._get_flavor_id({
                             "id": 'created_flavor_id',
                             "vcpus": 1,
                             "ram": 512,
                             "root_disk": 1,
                             "ephemeral_disk": 1,
                             "swap_disk": 1
                         }))

    @mock.patch('sahara_tests.scenario.base.BaseTestCase.addCleanup')
    @mock.patch('novaclient.v2.flavors.FlavorManager.create',
                return_value=FakeFlavor(flavor_id='created_flavor_id'))
    @mock.patch('novaclient.v2.flavors.FlavorManager.list',
                return_value=[FakeFlavor(flavor_id='existing_flavor_id',
                                         name='test-flavor')])
    def test_get_flavor_name_found(self, mock_list_flavor, mock_create_flavor,
                                   mock_base):
        self.base_scenario._init_clients()
        self.assertEqual('existing_flavor_id',
                         self.base_scenario._get_flavor_id({
                             'name': 'test-flavor',
                             "id": 'created_flavor_id',
                             "vcpus": 1,
                             "ram": 512,
                             "root_disk": 1,
                             "ephemeral_disk": 1,
                             "swap_disk": 1
                         }))

    @mock.patch('sahara_tests.scenario.base.BaseTestCase.addCleanup')
    @mock.patch('novaclient.v2.flavors.FlavorManager.create',
                return_value=FakeFlavor(flavor_id='created_flavor_id'))
    @mock.patch('novaclient.v2.flavors.FlavorManager.list',
                return_value=[FakeFlavor(flavor_id='another_flavor_id',
                                         name='another-flavor')])
    def test_get_flavor_id_not_found(self, mock_list_flavor,
                                     mock_create_flavor, mock_base):
        self.base_scenario._init_clients()
        self.assertEqual('created_flavor_id',
                         self.base_scenario._get_flavor_id({
                             'name': 'test-flavor',
                             "id": 'created_flavor_id',
                             "vcpus": 1,
                             "ram": 512,
                             "root_disk": 1,
                             "ephemeral_disk": 1,
                             "swap_disk": 1
                         }))

    @mock.patch('sahara_tests.scenario.base.BaseTestCase._run_command_on_node')
    def test_create_hdfs_data(self, mock_ssh):
        self.base_scenario._init_clients()
        output_path = '/user/test/data/output'
        self.assertEqual(output_path,
                         self.base_scenario._create_dfs_data(None, output_path,
                                                             None, 'hdfs'))
        input_path = ('sahara_tests/scenario/defaults/edp-examples/edp-pig/'
                      'trim-spaces/data/input')
        with mock.patch(
            'sahara_tests.scenario.clients.SaharaClient.get_cluster',
            return_value=FakeResponse(node_groups=[
                {
                    'node_processes': ['master', 'namenode'],
                    'instances': [{
                        'management_ip': 'test_ip'
                    }]
                }])):
            self.assertIn('/user/test/data-', (
                self.base_scenario._create_dfs_data(input_path, None,
                                                    'test', 'hdfs')))

    @mock.patch('saharaclient.api.base.ResourceManager._get',
                return_value=FakeResponse(
                    set_status=base.CLUSTER_STATUS_ACTIVE,
                    verification={'verification': {
                        'status': 'GREEN',
                        'cluster_id': 'id_cluster'
                    }}))
    @mock.patch('saharaclient.api.clusters.ClusterManager.verification_update')
    @mock.patch('sahara_tests.scenario.base.BaseTestCase.'
                'check_feature_available', return_value=True)
    def test_check_verification_did_not_start(self, mock_feature,
                                              mock_verification,
                                              mock_get_status):
        self.base_scenario._init_clients()
        self.assertIsNone(self.base_scenario.check_verification('id_cluster'))

    @mock.patch('saharaclient.api.base.ResourceManager._get',
                return_value=FakeResponse(
                    set_status=base.CLUSTER_STATUS_ACTIVE))
    @mock.patch('saharaclient.api.clusters.ClusterManager.verification_update')
    @mock.patch('sahara_tests.scenario.base.BaseTestCase.'
                'check_feature_available', return_value=True)
    @mock.patch('sahara_tests.scenario.base.BaseTestCase._get_health_status',
                return_value='GREEN')
    def test_verification_start(self, mock_status, mock_feature,
                                mock_verification, mock_get_status):
        self.base_scenario._init_clients()
        self.assertIsNone(self.base_scenario.check_verification('id_cluster'))

    @mock.patch('saharaclient.api.base.ResourceManager._get',
                return_value=FakeResponse(
                    set_status=base.CLUSTER_STATUS_ACTIVE))
    @mock.patch('saharaclient.api.clusters.ClusterManager.verification_update')
    def test_verification_skipped(self, mock_verification, mock_get_status):
        self.base_scenario._init_clients()
        self.assertIsNone(self.base_scenario.check_verification('id_cluster'))



# Copyright (c) 2015 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import re

import jsonschema
import rfc3986


SCHEMA = {
    "type": "object",
    "properties": {
        "concurrency": {
            "type": "integer",
            "minimum": 1
        },
        "credentials": {
            "type": "object",
            "properties": {
                "os_username": {
                    "type": "string",
                    "minLength": 1
                },
                "os_password": {
                    "type": "string",
                    "minLength": 1
                },
                "os_tenant": {
                    "type": "string",
                    "minLength": 1
                },
                "os_auth_url": {
                    "type": "string",
                    "format": "uri"
                },
                "sahara_service_type": {
                    "type": "string",
                    "minLength": 1
                },
                "sahara_url": {
                    "type": "string",
                    "format": "uri"
                },
                "ssl_verify": {
                    "type": "boolean"
                },
                "ssl_cert": {
                    "type": "string",
                    "minLength": 1
                },
            },
            "additionalProperties": False
        },
        "network": {
            "type": "object",
            "properties": {
                "type": {
                    "type": "string",
                    "enum": ["neutron", "nova-network"]
                },
                "auto_assignment_floating_ip": {
                    "type": "boolean"
                },
                "public_network": {
                    "type": "string",
                    "minLength": 1
                },
                "private_network": {
                    "type": "string",
                    "minLength": 1
                }
            },
            "additionalProperties": False
        },
        "clusters": {
            "type": "array",
            "minItems": 1,
            "items": {
                "type": "object",
                "properties": {
                    "existing_cluster": {
                        "type": "string",
                        "minLength": 1
                    },
                    "key_name": {
                        "type": "string",
                        "minLength": 1
                    },
                    "plugin_name": {
                        "type": "string",
                        "minLength": 1
                    },
                    "plugin_version": {
                        "type": "string",
                        "minLength": 1
                    },
                    "image": {
                        "type": "string",
                        "minLength": 1
                    },
                    "image_username": {
                        "type": "string",
                        "minLength": 1
                    },
                    "hdfs_username": {
                        "type": "string",
                        "minLength": 1
                    },
                    "node_group_templates": {
                        "type": "array",
                        "minItems": 1,
                        "items": {
                            "type": "object",
                            "properties": {
                                "name": {
                                    "type": "string",
                                    "minLength": 1,
                                    "format": "valid_name"
                                },
                                "node_processes": {
                                    "type": "array",
                                    "minItems": 1,
                                    "items": {
                                        "type": "string",
                                        "minLength": 1
                                    }
                                },
                                "flavor": {
                                    "type": ["object", "string"],
                                    "properties": {
                                        "name": {
                                            "type": "string",
                                            "minLength": 1
                                        },
                                        "id": {
                                            "type": "string",
                                            "minLength": 1
                                        },
                                        "vcpus": {
                                            "type": "integer",
                                            "minimum": 1
                                        },
                                        "ram": {
                                            "type": "integer",
                                            "minimum": 1
                                        },
                                        "root_disk": {
                                            "type": "integer",
                                            "minimum": 0
                                        },
                                        "ephemeral_disk": {
                                            "type": "integer",
                                            "minimum": 0
                                        },
                                        "swap_disk": {
                                            "type": "integer",
                                            "minimum": 0
                                        },
                                    },
                                    "additionalProperties": True
                                },
                                "description": {
                                    "type": "string"
                                },
                                "volumes_per_node": {
                                    "type": "integer",
                                    "minimum": 0
                                },
                                "volumes_size": {
                                    "type": "integer",
                                    "minimum": 0
                                },
                                "node_configs": {
                                    "type": "object"
                                },
                                "security_groups": {
                                    "type": "array",
                                    "items": {
                                        "type": "string",
                                        "minLength": 1
                                    }
                                },
                                "auto_security_group": {
                                    "type": "boolean"
                                },
                                "availability_zone": {
                                    "type": "string",
                                    "minLength": 1
                                },
                                "volumes_availability_zone": {
                                    "type": "string",
                                    "minLength": 1
                                },
                                "volume_type": {
                                    "type": "string",
                                    "minLength": 1
                                },
                                "is_proxy_gateway": {
                                    "type": "boolean"
                                }
                            },
                            "required": ["name", "flavor", "node_processes"],
                            "additionalProperties": False
                        }
                    },
                    "cluster_template": {
                        "type": "object",
                        "properties": {
                            "name": {
                                "type": "string",
                                "minLength": 1,
                                "format": "valid_name"
                            },
                            "description": {
                                "type": "string"
                            },
                            "cluster_configs": {
                                "type": "object"
                            },
                            "node_group_templates": {
                                "type": "object",
                                "patternProperties": {
                                    ".*": {
                                        "type": "integer",
                                        "minimum": 1
                                    }
                                }
                            },
                            "anti_affinity": {
                                "type": "array",
                                "items": {
                                    "type": "string",
                                    "minLength": 1
                                }
                            }
                        },
                        "required": ["node_group_templates"],
                        "additionalProperties": False
                    },
                    "cluster": {
                        "type": "object",
                        "properties": {
                            "name": {
                                "type": "string",
                                "minLength": 1,
                                "format": "valid_name"
                            },
                            "description": {
                                "type": "string"
                            },
                            "is_transient": {
                                "type": "boolean"
                            }
                        },
                        "additionalProperties": False,
                    },
                    "timeout_check_transient": {
                        "type": "integer",
                        "minimum": 1
                    },
                    "timeout_delete_resource": {
                        "type": "integer",
                        "minimum": 1
                    },
                    "timeout_poll_cluster_status": {
                        "type": "integer",
                        "minimum": 1
                    },
                    "timeout_poll_jobs_status": {
                        "type": "integer",
                        "minimum": 1
                    },
                    "custom_checks": {
                        "type": "object",
                        "properties": {
                            ".*": {
                                "type": "object",
                            }
                        }
                    },
                    "scaling": {
                        "type": "array",
                        "minItems": 1,
                        "items": {
                            "type": "object",
                            "properties": {
                                "operation": {
                                    "type": "string",
                                    "enum": ["add", "resize"]
                                },
                                "node_group": {
                                    "type": "string",
                                    "minLength": 1
                                },
                                "size": {
                                    "type": "integer",
                                    "minimum": 0
                                }
                            },
                            "required": ["operation", "node_group", "size"],
                            "additionalProperties": False
                        }
                    },
                    "scenario": {
                        "type": "array",
                        "items": {
                            "type": "string",
                            "minLength": 1
                        }
                    },
                    "edp_jobs_flow": {
                        "type": ["string", "array"]
                    },
                    "retain_resources": {
                        "type": "boolean"
                    },
                    "edp_batching": {
                        "type": "integer",
                        "minimum": 1
                    }
                },
                "required": ["plugin_name", "plugin_version", "image"],
                "additionalProperties": False
            }
        },
        "edp_jobs_flow": {
            "type": "object",
            "patternProperties": {
                ".*": {
                    "type": "array",
                    "minItems": 1,
                    "items": {
                        "type": "object",
                        "properties": {
                            "type": {
                                "type": "string",
                                "enum": ["Pig", "Java", "MapReduce",
                                         "MapReduce.Streaming", "Hive",
                                         "Spark"]
                            },
                            "input_datasource": {
                                "type": "object",
                                "properties": {
                                    "type": {
                                        "type": "string",
                                        "enum": ["swift", "hdfs", "maprfs"]
                                    },
                                    "source": {
                                        "type": "string"
                                    }
                                },
                                "required": ["type", "source"],
                                "additionalProperties": False
                            },
                            "output_datasource": {
                                "type": "object",
                                "properties": {
                                    "type": {
                                        "type": "string",
                                        "enum": ["swift", "hdfs", "maprfs"]
                                    },
                                    "destination": {
                                        "type": "string"
                                    }
                                },
                                "required": ["type", "destination"],
                                "additionalProperties": False
                            },
                            "main_lib": {
                                "type": "object",
                                "properties": {
                                    "type": {
                                        "type": "string",
                                        "enum": ["swift", "database"]
                                    },
                                    "source": {
                                        "type": "string"
                                    }
                                },
                                "required": ["type", "source"],
                                "additionalProperties": False
                            },
                            "additional_libs": {
                                "type": "array",
                                "items": {
                                    "type": "object",
                                    "properties": {
                                        "type": {
                                            "type": "string",
                                            "enum": ["swift", "database"]
                                        },
                                        "source": {
                                            "type": "string"
                                        }
                                    },
                                    "required": ["type", "source"],
                                    "additionalProperties": False
                                }
                            },
                            "configs": {
                                "type": "object"
                            },
                            "args": {
                                "type": "array"
                            }
                        },
                        "required": ["type"],
                        "additionalProperties": False
                    }
                }
            }
        }
    },
    "required": ["clusters"],
    "additionalProperties": False
}


@jsonschema.FormatChecker.cls_checks("uri")
def validate_uri_format(entry):
    return rfc3986.is_valid_uri(entry)


@jsonschema.FormatChecker.cls_checks('valid_name')
def validate_name_hostname_format(entry):
    res = re.match(r"^(([a-zA-Z]|[a-zA-Z][a-zA-Z0-9\-]"
                   r"*[a-zA-Z0-9])\.)*([A-Za-z]|[A-Za-z]"
                   r"[A-Za-z0-9\-]*[A-Za-z0-9])$", entry)
    return res is not None


class Validator(jsonschema.Draft4Validator):
    def __init__(self, schema):
        format_checker = jsonschema.FormatChecker()
        super(Validator, self).__init__(
            schema, format_checker=format_checker)


def validate(config):
    return Validator(SCHEMA).validate(config)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from os import fdopen
from os import path
from os import remove
import tempfile

from tempest.lib.common.utils import data_utils

from sahara_tempest_plugin.tests.cli import base


class SaharaJobBinaryCLITest(base.ClientTestBase):

    def openstack_job_binary_list(self):
        self.assertTableStruct(self.listing_result('job binary list'), [
            'Name',
            'Id',
            'Url'
        ])

    def openstack_job_binary_create(self):
        fd, script_name = tempfile.mkstemp()
        with fdopen(fd, 'w+') as jb:
            jb.write('test-script')
        job_binary_name = data_utils.rand_name('job-fake')
        flag = ("%(jb_name)s %(data)s "
                % {'jb_name': ('--name %s' % job_binary_name),
                   'data': ' --data %s' % script_name})
        self.assertTableStruct(
            self.listing_result('job binary create %s' % flag),
            [
                'Field',
                'Value'
            ])
        remove(script_name)
        return job_binary_name

    def openstack_job_binary_download(self, job_binary_name):
        if path.exists(job_binary_name):
            remove(job_binary_name)

        self.openstack('dataprocessing job binary download',
                       params=job_binary_name)

        self.assertTrue(path.exists(job_binary_name))
        remove(job_binary_name)

    def openstack_job_binary_show(self, job_binary_name):
        self.find_in_listing(self.listing_result('job binary show %s'
                                                 % job_binary_name),
                             job_binary_name)

    def openstack_job_binary_update(self, job_binary_name, flag=None):
        cmd = 'job binary update --%s' % flag
        if flag == 'description':
            self.assertTableStruct(
                self.listing_result('%s cli-tests %s'
                                    % (cmd, job_binary_name)), [
                    'Field',
                    'Value'
                ])
        elif flag == 'name':
            new_job_binary_name = data_utils.rand_name(job_binary_name)
            self.assertTableStruct(
                self.listing_result('%s %s %s'
                                    % (cmd, new_job_binary_name,
                                       job_binary_name)), [
                    'Field',
                    'Value'
                ])
            return new_job_binary_name
        else:
            # here we check only updating with public/protected flags for now
            self.assertTableStruct(
                self.listing_result('%s %s' % (cmd, job_binary_name)), [
                    'Field',
                    'Value'
                ]
            )

    def openstack_job_binary_delete(self, job_binary_name):
        self.openstack('dataprocessing job binary delete',
                       params=job_binary_name)
        list_job_binary = self.listing_result('job binary list')
        for job_binary in list_job_binary:
            if job_binary['Name'] == job_binary_name:
                raise self.skipException('Job binary is not delete')

    def negative_delete_removed_job_binary(self, job_binary_name):
        """Test to remove already deleted job binary"""
        command_to_execute = 'job binary delete'
        self.check_negative_scenarios(base.TEMPEST_ERROR_MESSAGE,
                                      command_to_execute,
                                      job_binary_name)

    def negative_try_to_update_protected_jb(self, job_binary_name):
        """Test to try to update proteted job binary"""
        self.openstack_job_binary_update(job_binary_name, flag='protected')
        error_message = ("JobBinary with id '%s' could not be updated because "
                         "it's marked as protected" %
                         self._get_resource_id('job binary',
                                               job_binary_name))
        command_to_execute = 'job binary update --name test'
        self.check_negative_scenarios(error_message,
                                      command_to_execute,
                                      job_binary_name)

    def filter_job_binaries_in_list(self):
        """Filter job binaries list with --column flag"""
        job_binaries_list = self.listing_result('job binary list '
                                                '--column Name')
        self.assertTableStruct(job_binaries_list, ['Name'])



# Copyright 2016 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import os

from tempest import config
from tempest.test_discover import plugins

from sahara_tempest_plugin import config as sahara_config


class SaharaTempestPlugin(plugins.TempestPlugin):
    def load_tests(self):
        base_path = os.path.split(os.path.dirname(
            os.path.abspath(__file__)))[0]
        test_dir = "sahara_tempest_plugin/tests"
        full_test_dir = os.path.join(base_path, test_dir)
        return full_test_dir, base_path

    def register_opts(self, conf):
        conf.register_opt(sahara_config.service_option,
                          group='service_available')
        conf.register_group(sahara_config.data_processing_group)
        conf.register_opts(sahara_config.DataProcessingGroup +
                           sahara_config.DataProcessingAdditionalGroup,
                           sahara_config.data_processing_group)

        conf.register_group(sahara_config.data_processing_feature_group)
        conf.register_opts(sahara_config.DataProcessingFeaturesGroup,
                           sahara_config.data_processing_feature_group)

    def get_opt_lists(self):
        return [
            (sahara_config.data_processing_group.name,
             sahara_config.DataProcessingGroup),
            (sahara_config.data_processing_feature_group.name,
             sahara_config.DataProcessingFeaturesGroup),
        ]

    def get_service_clients(self):
        data_processing_config = (
            config.service_client_config('data-processing'))

        params = {
            'name': 'data_processing',
            'service_version': 'data_processing.v1_1',
            'module_path':
                'sahara_tempest_plugin.services.data_processing.v1_1',
            'client_names': ['DataProcessingClient']
        }
        params.update(data_processing_config)
        return [params]



# Copyright (c) 2014 Mirantis Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from tempest import config
from tempest.lib import exceptions
import tempest.test

from sahara_tempest_plugin import clients
from sahara_tempest_plugin.common import plugin_utils


CONF = config.CONF


class InvalidSaharaTestConfiguration(exceptions.TempestException):
    message = "Invalid configuration for Sahara tests"


class BaseDataProcessingTest(tempest.test.BaseTestCase):

    credentials = ['primary']

    client_manager = clients.Manager

    @classmethod
    def skip_checks(cls):
        super(BaseDataProcessingTest, cls).skip_checks()
        if not CONF.service_available.sahara:
            raise cls.skipException('Sahara support is required')
        cls.default_plugin = plugin_utils.get_default_plugin()

    @classmethod
    def setup_clients(cls):
        super(BaseDataProcessingTest, cls).setup_clients()
        cls.client = cls.os_primary.data_processing.DataProcessingClient()

    @classmethod
    def resource_setup(cls):
        super(BaseDataProcessingTest, cls).resource_setup()

        plugin = None
        if cls.default_plugin:
            plugin = cls.client.get_plugin(cls.default_plugin)['plugin']
        cls.default_version = plugin_utils.get_default_version(plugin)

        if cls.default_plugin is not None and cls.default_version is None:
            raise InvalidSaharaTestConfiguration(
                message="No known Sahara plugin version was found")

        # add lists for watched resources
        cls._node_group_templates = []
        cls._cluster_templates = []
        cls._data_sources = []
        cls._job_binary_internals = []
        cls._job_binaries = []
        cls._jobs = []

    @classmethod
    def resource_cleanup(cls):
        cls.cleanup_resources(getattr(cls, '_cluster_templates', []),
                              cls.client.delete_cluster_template)
        cls.cleanup_resources(getattr(cls, '_node_group_templates', []),
                              cls.client.delete_node_group_template)
        cls.cleanup_resources(getattr(cls, '_jobs', []), cls.client.delete_job)
        cls.cleanup_resources(getattr(cls, '_job_binaries', []),
                              cls.client.delete_job_binary)
        cls.cleanup_resources(getattr(cls, '_job_binary_internals', []),
                              cls.client.delete_job_binary_internal)
        cls.cleanup_resources(getattr(cls, '_data_sources', []),
                              cls.client.delete_data_source)
        super(BaseDataProcessingTest, cls).resource_cleanup()

    @staticmethod
    def cleanup_resources(resource_id_list, method):
        for resource_id in resource_id_list:
            try:
                method(resource_id)
            except exceptions.NotFound:
                # ignore errors while auto removing created resource
                pass

    @classmethod
    def create_node_group_template(cls, name, plugin_name, hadoop_version,
                                   node_processes, flavor_id,
                                   node_configs=None, **kwargs):
        """Creates watched node group template with specified params.

        It supports passing additional params using kwargs and returns created
        object. All resources created in this method will be automatically
        removed in tearDownClass method.
        """
        resp_body = cls.client.create_node_group_template(name, plugin_name,
                                                          hadoop_version,
                                                          node_processes,
                                                          flavor_id,
                                                          node_configs,
                                                          **kwargs)
        resp_body = resp_body['node_group_template']
        # store id of created node group template
        cls._node_group_templates.append(resp_body['id'])

        return resp_body

    @classmethod
    def create_cluster_template(cls, name, plugin_name, hadoop_version,
                                node_groups, cluster_configs=None, **kwargs):
        """Creates watched cluster template with specified params.

        It supports passing additional params using kwargs and returns created
        object. All resources created in this method will be automatically
        removed in tearDownClass method.
        """
        resp_body = cls.client.create_cluster_template(name, plugin_name,
                                                       hadoop_version,
                                                       node_groups,
                                                       cluster_configs,
                                                       **kwargs)
        resp_body = resp_body['cluster_template']
        # store id of created cluster template
        cls._cluster_templates.append(resp_body['id'])

        return resp_body

    @classmethod
    def create_data_source(cls, name, type, url, **kwargs):
        """Creates watched data source with specified params.

        It supports passing additional params using kwargs and returns created
        object. All resources created in this method will be automatically
        removed in tearDownClass method.
        """
        resp_body = cls.client.create_data_source(name, type, url, **kwargs)
        resp_body = resp_body['data_source']
        # store id of created data source
        cls._data_sources.append(resp_body['id'])

        return resp_body

    @classmethod
    def create_job_binary_internal(cls, name, data):
        """Creates watched job binary internal with specified params.

        It returns created object. All resources created in this method will
        be automatically removed in tearDownClass method.
        """
        resp_body = cls.client.create_job_binary_internal(name, data)
        resp_body = resp_body['job_binary_internal']
        # store id of created job binary internal
        cls._job_binary_internals.append(resp_body['id'])

        return resp_body

    @classmethod
    def create_job_binary(cls, name, url, extra=None, **kwargs):
        """Creates watched job binary with specified params.

        It supports passing additional params using kwargs and returns created
        object. All resources created in this method will be automatically
        removed in tearDownClass method.
        """
        resp_body = cls.client.create_job_binary(name, url, extra, **kwargs)
        resp_body = resp_body['job_binary']
        # store id of created job binary
        cls._job_binaries.append(resp_body['id'])

        return resp_body

    @classmethod
    def create_job(cls, name, job_type, mains, libs=None, **kwargs):
        """Creates watched job with specified params.

        It supports passing additional params using kwargs and returns created
        object. All resources created in this method will be automatically
        removed in tearDownClass method.
        """
        resp_body = cls.client.create_job(name,
                                          job_type, mains, libs, **kwargs)
        resp_body = resp_body['job']
        # store id of created job
        cls._jobs.append(resp_body['id'])

        return resp_body

    @classmethod
    def get_node_group_template(cls, nodegroup='worker1'):
        """Returns a node group template for the default plugin."""
        return plugin_utils.get_node_group_template(nodegroup,
                                                    cls.default_version)

    @classmethod
    def get_cluster_template(cls, node_group_template_ids=None):
        """Returns a cluster template for the default plugin.

        node_group_template_defined contains the type and ID of pre-defined
        node group templates that have to be used in the cluster template
        (instead of dynamically defining them with 'node_processes').
        """
        return plugin_utils.get_cluster_template(node_group_template_ids,
                                                 cls.default_version)

    @classmethod
    def wait_for_resource_deletion(cls, resource_id, get_resource):
        """Waits for a resource to be deleted.

        The deletion of a resource depends on the client is_resource_deleted
        implementation. This implementation will vary slightly from resource
        to resource. get_resource param should be the function used to
        retrieve that type of resource.
        """
        def is_resource_deleted(resource_id):
            try:
                get_resource(resource_id)
            except exceptions.NotFound:
                return True
            return False

        cls.client.is_resource_deleted = is_resource_deleted
        cls.client.wait_for_resource_deletion(resource_id)
